@@ -30,6 +30,9 @@
 
 #include "gstgdkpixbuf.h"
 
+GST_DEBUG_CATEGORY_STATIC (gst_gdk_pixbuf_debug);
+#define GST_CAT_DEFAULT gst_gdk_pixbuf_debug
+
 static GstElementDetails plugin_details = {
   "GdkPixbuf image decoder",
   "Codec/Decoder/Image",
@@ -95,7 +98,9 @@ static void	gst_gdk_pixbuf_get_property(GObject *object, guint prop_id,
 						 GParamSpec *pspec);
 
 static void	gst_gdk_pixbuf_chain	(GstPad *pad, GstData *_data);
+#ifdef enable_typefind
 static void     gst_gdk_pixbuf_type_find (GstTypeFind *tf, gpointer ignore);
+#endif
 
 static GstElementClass *parent_class = NULL;
 
@@ -109,7 +114,7 @@ gst_gdk_pixbuf_sink_link (GstPad *pad, const GstCaps *caps)
   g_return_val_if_fail (GST_IS_GDK_PIXBUF (filter),
                         GST_PAD_LINK_REFUSED);
 
-  filter->framerate = 0.0;
+  filter->framerate = 1.0;
   gst_structure_get_double (gst_caps_get_structure (caps, 0), "framerate",
       &filter->framerate);
 
@@ -285,9 +290,9 @@ gst_gdk_pixbuf_chain (GstPad *pad, GstData *_data)
     if (filter->pixbuf_loader != NULL) {
       GstBuffer *outbuf;
       GdkPixbuf *pixbuf;
-      GError *error;
+      GError *error = NULL;
 
-      if (gdk_pixbuf_loader_close (filter->pixbuf_loader, &error)) {
+      if (!gdk_pixbuf_loader_close (filter->pixbuf_loader, &error)) {
         GST_ELEMENT_ERROR (filter, LIBRARY, SHUTDOWN, (NULL), (error->message));
         g_error_free (error);
         return;
@@ -393,6 +398,7 @@ gst_gdk_pixbuf_get_property (GObject *object, guint prop_id,
 
 #define GST_GDK_PIXBUF_TYPE_FIND_SIZE 1024
 
+#ifdef enable_typefind
 static void
 gst_gdk_pixbuf_type_find (GstTypeFind *tf, gpointer ignore)
 {
@@ -403,7 +409,7 @@ gst_gdk_pixbuf_type_find (GstTypeFind *tf, gpointer ignore)
   data = gst_type_find_peek (tf, 0, GST_GDK_PIXBUF_TYPE_FIND_SIZE);
   if (data == NULL) return;
 
-  GST_DEBUG ("gst_gdk_pixbuf_type_find");
+  GST_DEBUG ("creating new loader");
 
   pixbuf_loader = gdk_pixbuf_loader_new();
   
@@ -414,17 +420,28 @@ gst_gdk_pixbuf_type_find (GstTypeFind *tf, gpointer ignore)
 
   if (format != NULL) {
     GstCaps *caps;
-    gchar **mlist = gdk_pixbuf_format_get_mime_types(format);
+    gchar **p;
+    gchar **mlist = gdk_pixbuf_format_get_mime_types (format);
 
-    caps = gst_caps_new_simple (mlist[0], NULL);
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MINIMUM, caps);
-    gst_caps_free (caps);
+    for (p = mlist; *p; ++p)
+    {
+      GST_DEBUG ("suggesting mime type %s", *p);
+      caps = gst_caps_new_simple (*p, NULL);
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MINIMUM, caps);
+      gst_caps_free (caps);
+    }
     g_strfreev (mlist);
   }
 
+  GST_DEBUG ("closing pixbuf loader, hope it doesn't hang ...");
+  /* librsvg 2.4.x has a bug where it triggers an endless loop in trying
+     to close a gzip that's not an svg; fixed upstream but no good way
+     to work around it */
   gdk_pixbuf_loader_close (pixbuf_loader, NULL);
+  GST_DEBUG ("closed pixbuf loader");
   g_object_unref (G_OBJECT (pixbuf_loader));
 }
+#endif
 
 /* entry point to initialize the plug-in
  * initialize the plug-in itself
@@ -434,16 +451,16 @@ gst_gdk_pixbuf_type_find (GstTypeFind *tf, gpointer ignore)
 static gboolean
 plugin_init (GstPlugin *plugin)
 {
-  GstCaps *caps;
-  
+  GST_DEBUG_CATEGORY_INIT (gst_gdk_pixbuf_debug, "gdkpixbuf", 0, "gdk pixbuf loader");
+
   if (!gst_element_register (plugin, "gdkpixbufdec", GST_RANK_NONE, GST_TYPE_GDK_PIXBUF))
     return FALSE;
 
-  caps = GST_CAPS_ANY;
+#ifdef enable_typefind
   gst_type_find_register (plugin, "image/*", GST_RANK_MARGINAL,
 			  gst_gdk_pixbuf_type_find, NULL,
-                          caps, NULL);
-  gst_caps_free (caps);
+                          GST_CAPS_ANY, NULL);
+#endif
 
   /* plugin initialisation succeeded */
   return TRUE;
