@@ -502,6 +502,16 @@ gst_mad_id3_to_tag_list(const struct id3_tag *tag)
 
 	      gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, GST_TAG_TRACK_COUNT, total, NULL);
 	    }
+	  } else if (strcmp(tag_name,GST_TAG_ALBUM_VOLUME_NUMBER) == 0) {
+	    if (*check == '/') {
+	      guint total;
+
+	      check++;
+	      total = strtoul (check, &check, 10);
+	      if (*check != '\0') break;
+
+	      gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, GST_TAG_ALBUM_VOLUME_COUNT, total, NULL);
+	    }
 	  }
 
 	  if (*check != '\0') break;		
@@ -871,11 +881,9 @@ gst_id3_tag_chain (GstPad *pad, GstData *data)
 	  }
 	} else {
 	  GST_WARNING_OBJECT (tag, "detected ID3v1 tag, but couldn't parse it");
-	  tag->v2tag_size = 0;
 	}
       } else if (tag->v1tag_size != 0) {
 	GST_WARNING_OBJECT (tag, "bad non-ID3v1 tag at end of file");
-	tag->v1tag_size = 0;
       } else {
 	GST_LOG_OBJECT (tag, "no ID3v1 tag (%"G_GUINT64_FORMAT")", GST_BUFFER_OFFSET (tag->buffer));
       }
@@ -933,7 +941,6 @@ gst_id3_tag_chain (GstPad *pad, GstData *data)
 	  tag->parsed_tags = list;
 	} else {
 	  GST_WARNING_OBJECT (tag, "detected ID3v2 tag, but couldn't parse it");
-	  tag->v2tag_size = 0;
 	}
       }
       /* caps nego and typefinding */
