@@ -31,6 +31,8 @@
 #if MPEG2_RELEASE < MPEG2_VERSION(0,3,2)
 */
 #ifndef MPEG2_RELEASE
+#define MPEG2_VERSION(a,b,c) ((((a)&0xff)<<16)|(((b)&0xff)<<8)|((c)&0xff))
+#define MPEG2_RELEASE MPEG2_VERSION(0,3,1)
 typedef picture_t mpeg2_picture_t;
 typedef gint mpeg2_state_t;
 #define STATE_BUFFER 0
@@ -54,59 +56,40 @@ enum {
 
 enum {
   ARG_0,
-  ARG_STREAMINFO,
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (src_template_factory,
+static GstStaticPadTemplate src_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "mpeg2dec_src",
-    "video/x-raw-yuv",
-      "format",       GST_PROPS_LIST (
-	GST_PROPS_FOURCC (GST_MAKE_FOURCC ('Y','V','1','2')),
-	GST_PROPS_FOURCC (GST_MAKE_FOURCC ('I','4','2','0'))
-		      ),
-      "width",        GST_PROPS_INT_RANGE (16, 4096),
-      "height",       GST_PROPS_INT_RANGE (16, 4096),
-      "pixel_width",  GST_PROPS_INT_RANGE (1, 255),
-      "pixel_height", GST_PROPS_INT_RANGE (1, 255),
-      "framerate",    GST_PROPS_LIST (
-			GST_PROPS_FLOAT (24/1.001),
-			GST_PROPS_FLOAT (24.),
-			GST_PROPS_FLOAT (25.),
-			GST_PROPS_FLOAT (30/1.001),
-			GST_PROPS_FLOAT (30.),
-			GST_PROPS_FLOAT (50.),
-			GST_PROPS_FLOAT (60/1.001),
-			GST_PROPS_FLOAT (60.)
-		      )
-  )
+  GST_STATIC_CAPS ("video/x-raw-yuv, "
+      "format = (fourcc) { YV12, I420 }, "
+      "width = (int) [ 16, 4096 ], "
+      "height = (int) [ 16, 4096 ], "
+      "pixel_width = (int) [ 1, 255 ], "
+      "pixel_height = (int) [ 1, 255 ], "
+      "framerate = (double) { 23.976024, 24.0, "
+        "25.0, 29.970030, 30.0, 50.0, 59.940060, 60.0 }")
 );
 
-GST_PAD_TEMPLATE_FACTORY (user_data_template_factory,
+static GstStaticPadTemplate user_data_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "user_data",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "mpeg2dec_user_data",
-    "application/octet-stream",
-    NULL
-  )
+  GST_STATIC_CAPS_ANY
 );
 
-GST_PAD_TEMPLATE_FACTORY (sink_template_factory,
+static GstStaticPadTemplate sink_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "mpeg2dec_sink",
-    "video/mpeg",
-      /* width/height/framerate not needed */
-      "mpegversion",  GST_PROPS_INT_RANGE (1, 2),
-      "systemstream", GST_PROPS_BOOLEAN (FALSE)
+  GST_STATIC_CAPS ("video/mpeg, "
+    "mpegversion = (int) [ 1, 2 ], "
+    "systemstream = (boolean) false"
   )
 );
 
@@ -175,9 +158,9 @@ gst_mpeg2dec_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (src_template_factory));
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (sink_template_factory));
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (user_data_template_factory));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get (&src_template_factory));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get (&sink_template_factory));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get (&user_data_template_factory));
 
   gst_element_class_set_details (element_class, &gst_mpeg2dec_details);
 }
@@ -193,10 +176,6 @@ gst_mpeg2dec_class_init(GstMpeg2decClass *klass)
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
 
-  g_object_class_install_property (gobject_class, ARG_STREAMINFO,
-    g_param_spec_boxed ("streaminfo", "Streaminfo", "Streaminfo",
-                        GST_TYPE_CAPS, G_PARAM_READABLE));
-  
   gobject_class->set_property 	= gst_mpeg2dec_set_property;
   gobject_class->get_property 	= gst_mpeg2dec_get_property;
   gobject_class->dispose 	= gst_mpeg2dec_dispose;
@@ -211,15 +190,16 @@ gst_mpeg2dec_init (GstMpeg2dec *mpeg2dec)
 {
   /* create the sink and src pads */
   mpeg2dec->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (sink_template_factory), "sink");
+		  gst_static_pad_template_get (&sink_template_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (mpeg2dec), mpeg2dec->sinkpad);
   gst_pad_set_chain_function (mpeg2dec->sinkpad, GST_DEBUG_FUNCPTR (gst_mpeg2dec_chain));
   gst_pad_set_formats_function (mpeg2dec->sinkpad, GST_DEBUG_FUNCPTR (gst_mpeg2dec_get_sink_formats));
   gst_pad_set_convert_function (mpeg2dec->sinkpad, GST_DEBUG_FUNCPTR (gst_mpeg2dec_convert_sink));
 
   mpeg2dec->srcpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (src_template_factory), "src");
+		  gst_static_pad_template_get (&src_template_factory), "src");
   gst_element_add_pad (GST_ELEMENT (mpeg2dec), mpeg2dec->srcpad);
+  gst_pad_use_explicit_caps (mpeg2dec->srcpad);
   gst_pad_set_formats_function (mpeg2dec->srcpad, GST_DEBUG_FUNCPTR (gst_mpeg2dec_get_src_formats));
   gst_pad_set_event_mask_function (mpeg2dec->srcpad, GST_DEBUG_FUNCPTR (gst_mpeg2dec_get_src_event_masks));
   gst_pad_set_event_function (mpeg2dec->srcpad, GST_DEBUG_FUNCPTR (gst_mpeg2dec_src_event));
@@ -228,7 +208,7 @@ gst_mpeg2dec_init (GstMpeg2dec *mpeg2dec)
   gst_pad_set_convert_function (mpeg2dec->srcpad, GST_DEBUG_FUNCPTR (gst_mpeg2dec_convert_src));
 
   mpeg2dec->userdatapad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (user_data_template_factory), "user_data");
+		  gst_static_pad_template_get (&user_data_template_factory), "user_data");
   gst_element_add_pad (GST_ELEMENT (mpeg2dec), mpeg2dec->userdatapad);
 
   /* initialize the mpeg2dec acceleration */
@@ -294,12 +274,7 @@ gst_mpeg2dec_alloc_buffer (GstMpeg2dec *mpeg2dec, const mpeg2_info_t *info, gint
   guint8 *buf[3], *out;
   const mpeg2_picture_t *picture;
 
-  if (mpeg2dec->peerpool) {
-    outbuf = gst_buffer_new_from_pool (mpeg2dec->peerpool, 0, 0);
-  }
-  if (!outbuf) {
-    outbuf = gst_buffer_new_and_alloc ((size * 3) / 2);
-  }
+  outbuf = gst_buffer_new_and_alloc ((size * 3) / 2);
 
   out = GST_BUFFER_DATA (outbuf);
 
@@ -334,74 +309,41 @@ gst_mpeg2dec_alloc_buffer (GstMpeg2dec *mpeg2dec, const mpeg2_info_t *info, gint
 static gboolean
 gst_mpeg2dec_negotiate_format (GstMpeg2dec *mpeg2dec)
 {
-  GstCaps *allowed;
-  GstCaps *intersect, *trylist, *head, *to_intersect;
+  GstCaps *caps;
+  guint32 fourcc;
+  gboolean ret;
 
   if (!GST_PAD_IS_LINKED (mpeg2dec->srcpad)) {
     mpeg2dec->format = MPEG2DEC_FORMAT_I420;
     return TRUE;
   }
 
-  /* we what we are allowed to do */
-  allowed = gst_pad_get_allowed_caps (mpeg2dec->srcpad);
-  /* we could not get allowed caps */
-  if (!allowed) {
-    allowed = GST_CAPS_NEW (
-                "mpeg2dec_negotiate",
-                "video/x-raw-yuv",
-               	  "format",        GST_PROPS_FOURCC (GST_STR_FOURCC ("I420"))
-	      );
-  }
+  caps = gst_caps_new_simple ("video/x-raw-yuv",
+      "format",       GST_TYPE_FOURCC, GST_STR_FOURCC ("I420"),
+      "width",        G_TYPE_INT, mpeg2dec->width,
+      "height",       G_TYPE_INT, mpeg2dec->height,
+      "pixel_width",  G_TYPE_INT, mpeg2dec->pixel_width,
+      "pixel_height", G_TYPE_INT, mpeg2dec->pixel_height,
+      "framerate",    G_TYPE_DOUBLE, 1. * GST_SECOND / mpeg2dec->frame_period,
+      NULL);
 
-  to_intersect = GST_CAPS_NEW (
-                   "mpeg2dec_negotiate",
-                      "video/x-raw-yuv",
-                      "width",        GST_PROPS_INT (mpeg2dec->width),
- 	              "height",       GST_PROPS_INT (mpeg2dec->height),
-                      "pixel_width",  GST_PROPS_INT (mpeg2dec->pixel_width),
- 	              "pixel_height", GST_PROPS_INT (mpeg2dec->pixel_height),
-		      "framerate",    GST_PROPS_FLOAT (1. * GST_SECOND / mpeg2dec->frame_period)
-                 );
+  ret = gst_pad_set_explicit_caps (mpeg2dec->srcpad, caps);
+  if (!ret) return FALSE;
 
-  /* try to fix our height */
-  intersect = gst_caps_intersect (allowed, to_intersect);
-  gst_caps_unref (allowed);
-  gst_caps_unref (to_intersect);
-
-  /* prepare for looping */
-  head = trylist = gst_caps_normalize (intersect);
-  gst_caps_unref (intersect);
-
-  while (trylist) {
-    GstCaps *to_try = gst_caps_copy_1 (trylist);
-
-    /* try each format */
-    if (gst_pad_try_set_caps (mpeg2dec->srcpad, to_try) > 0) {
-      guint32 fourcc;
-
-      /* it worked, try to find what it was again */
-      gst_caps_get_fourcc_int (to_try, "format", &fourcc);
+  /* it worked, try to find what it was again */
+  gst_structure_get_fourcc (gst_caps_get_structure (caps,0),
+      "format", &fourcc);
 								      
-      if (fourcc == GST_STR_FOURCC ("I420")) {
-	mpeg2dec->format = MPEG2DEC_FORMAT_I420;
-      }
-      else {
-	mpeg2dec->format = MPEG2DEC_FORMAT_YV12;
-      }
-      break;
-    }
-
-    trylist = trylist->next;
+  if (fourcc == GST_STR_FOURCC ("I420")) {
+    mpeg2dec->format = MPEG2DEC_FORMAT_I420;
+  } else {
+    mpeg2dec->format = MPEG2DEC_FORMAT_YV12;
   }
-  gst_caps_unref (head);
 
-  /* oops list exhausted and nothing was found... */
-  if (!trylist) {
-    return FALSE;
-  }
   return TRUE;
 }
 
+#if 0
 static void
 update_streaminfo (GstMpeg2dec *mpeg2dec)
 {
@@ -414,9 +356,9 @@ update_streaminfo (GstMpeg2dec *mpeg2dec)
 
   props = gst_props_empty_new ();
 
-  entry = gst_props_entry_new ("framerate", GST_PROPS_FLOAT (GST_SECOND/(float)mpeg2dec->frame_period));
+  entry = gst_props_entry_new ("framerate", G_TYPE_DOUBLE (GST_SECOND/(float)mpeg2dec->frame_period));
   gst_props_add_entry (props, entry);
-  entry = gst_props_entry_new ("bitrate", GST_PROPS_INT (info->sequence->byte_rate * 8));
+  entry = gst_props_entry_new ("bitrate", G_TYPE_INT (info->sequence->byte_rate * 8));
   gst_props_add_entry (props, entry);
 
   caps = gst_caps_new ("mpeg2dec_streaminfo",
@@ -426,6 +368,7 @@ update_streaminfo (GstMpeg2dec *mpeg2dec)
   gst_caps_replace_sink (&mpeg2dec->streaminfo, caps);
   g_object_notify (G_OBJECT (mpeg2dec), "streaminfo");
 }
+#endif
 
 static void
 gst_mpeg2dec_flush_decoder (GstMpeg2dec *mpeg2dec)
@@ -495,7 +438,11 @@ gst_mpeg2dec_chain (GstPad *pad, GstData *_data)
     GST_DEBUG ("have pts: %" G_GINT64_FORMAT " (%" G_GINT64_FORMAT ")", 
 		  mpeg_pts, MPEGTIME_TO_GSTTIME (mpeg_pts));
 
+#if MPEG2_RELEASE >= MPEG2_VERSION(0,4,0)
+    mpeg2_tag_picture (mpeg2dec->decoder, mpeg_pts&0xffffffff, mpeg_pts>>32);
+#else
     mpeg2_pts (mpeg2dec->decoder, mpeg_pts);
+#endif
   }
   else {
     GST_DEBUG ("no pts");
@@ -529,17 +476,10 @@ gst_mpeg2dec_chain (GstPad *pad, GstData *_data)
 		      info->sequence->transfer_characteristics, info->sequence->matrix_coefficients);
 
 	if (!gst_mpeg2dec_negotiate_format (mpeg2dec)) {
-          gst_element_error (GST_ELEMENT (mpeg2dec), "could not negotiate format");
+          GST_ELEMENT_ERROR (mpeg2dec, CORE, NEGOTIATION, (NULL), (NULL));
 	  goto exit;
 	}
 
-	/* now that we've negotiated, try to get a bufferpool */
-	mpeg2dec->peerpool = gst_pad_get_bufferpool (mpeg2dec->srcpad);
-	if (mpeg2dec->peerpool)
-	  GST_INFO ( "got pool %p", mpeg2dec->peerpool);
-
-	update_streaminfo (mpeg2dec);
-
 	if (!mpeg2dec->have_fbuf) {
 	  /* alloc 3 buffers */
 	  gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
@@ -592,6 +532,9 @@ gst_mpeg2dec_chain (GstPad *pad, GstData *_data)
         break;
       case STATE_SLICE:
 	slice = TRUE;
+#if MPEG2_RELEASE >= MPEG2_VERSION (0, 4, 0)
+      case STATE_INVALID_END:
+#endif
       case STATE_END:
       {
 	GstBuffer *outbuf = NULL;
@@ -619,18 +562,22 @@ gst_mpeg2dec_chain (GstPad *pad, GstData *_data)
 	  if (mpeg2dec->discont_state == MPEG2DEC_DISC_NEW_KEYFRAME && key_frame)
 	    mpeg2dec->discont_state = MPEG2DEC_DISC_NONE;
 
+#if MPEG2_RELEASE < MPEG2_VERSION(0,4,0)
 	  if (picture->flags & PIC_FLAG_PTS) {
             GstClockTime time = MPEGTIME_TO_GSTTIME (picture->pts);
+#else
+          if (picture->flags & PIC_FLAG_TAGS) {
+            GstClockTime time = MPEGTIME_TO_GSTTIME ((guint64) picture->tag2 << 32 | picture->tag);
+#endif
 
 	    GST_DEBUG ("picture had pts %" G_GINT64_FORMAT, time);
-            GST_BUFFER_TIMESTAMP (outbuf) = time;
-
-            mpeg2dec->next_time = time;
+            GST_BUFFER_TIMESTAMP (outbuf) = mpeg2dec->next_time = time;
 	  }
 	  else {
 	    GST_DEBUG ("picture didn't have pts using %" G_GINT64_FORMAT, mpeg2dec->next_time);
             GST_BUFFER_TIMESTAMP (outbuf) = mpeg2dec->next_time;
 	  }
+
           mpeg2dec->next_time += (mpeg2dec->frame_period * picture->nb_fields) >> 1;
 
 
@@ -918,6 +865,7 @@ gst_mpeg2dec_get_src_event_masks (GstPad *pad)
 {
   static const GstEventMask masks[] = {
     { GST_EVENT_SEEK, GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH },
+    { GST_EVENT_NAVIGATION, GST_EVENT_FLAG_NONE },
     { 0, }
   };
   return masks;
@@ -1068,6 +1016,9 @@ gst_mpeg2dec_src_event (GstPad *pad, GstEvent *event)
           mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_PICTURE;
       }
       break;
+    case GST_EVENT_NAVIGATION:
+      /* Forward a navigation event unchanged */
+      return gst_pad_send_event (GST_PAD_PEER (mpeg2dec->sinkpad), event);
     default:
       res = FALSE;
       break;
@@ -1087,7 +1038,6 @@ gst_mpeg2dec_change_state (GstElement *element)
     case GST_STATE_READY_TO_PAUSED:
     {
       mpeg2dec->next_time = 0;
-      mpeg2dec->peerpool = NULL;
 
       /* reset the initial video state */
       mpeg2dec->format = MPEG2DEC_FORMAT_NONE;
@@ -1097,29 +1047,16 @@ gst_mpeg2dec_change_state (GstElement *element)
       mpeg2dec->segment_end = -1;
       mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_PICTURE;
       mpeg2dec->frame_period = 0;
-      mpeg2dec->streaminfo = NULL;
       gst_mpeg2dec_open_decoder (mpeg2dec);
       mpeg2dec->need_sequence = TRUE;
       break;
     }
     case GST_STATE_PAUSED_TO_PLAYING:
-      /* if we've negotiated caps, try to get a bufferpool */
-      if (mpeg2dec->peerpool == NULL && mpeg2dec->width > 0) {
-	mpeg2dec->peerpool = gst_pad_get_bufferpool (mpeg2dec->srcpad);
-	if (mpeg2dec->peerpool)
-	  GST_INFO ( "got pool %p", mpeg2dec->peerpool);
-      }
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
-      /* need to clear things we get from other plugins, since we could be reconnected */
-      if (mpeg2dec->peerpool) {
-	gst_buffer_pool_unref (mpeg2dec->peerpool);
-	mpeg2dec->peerpool = NULL;
-      }
       break;
     case GST_STATE_PAUSED_TO_READY:
       gst_mpeg2dec_close_decoder (mpeg2dec);
-      gst_caps_replace (&mpeg2dec->streaminfo, NULL);
       break;
     case GST_STATE_READY_TO_NULL:
       break;
@@ -1155,9 +1092,6 @@ gst_mpeg2dec_get_property (GObject *object, guint prop_id, GValue *value, GParam
   mpeg2dec = GST_MPEG2DEC (object);
 
   switch (prop_id) {
-    case ARG_STREAMINFO:
-      g_value_set_boxed (value, mpeg2dec->streaminfo);
-      break;
     default:
       break;
   }
