@@ -19,21 +19,23 @@
 
 
 /*#define GST_DEBUG_ENABLED*/
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "gstmpegparse.h"
 #include "gstmpegclock.h"
 
 static GstFormat scr_format;
 
+GST_DEBUG_CATEGORY_EXTERN (GST_CAT_SEEK);
+
 /* elementfactory information */
 static GstElementDetails mpeg_parse_details = {
   "MPEG System Parser",
   "Codec/Parser",
-  "LGPL",
   "Parses MPEG1 and MPEG2 System Streams",
-  VERSION,
   "Erik Walthinsen <omega@cse.ogi.edu>\n"
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 1999",
+  "Wim Taymans <wim.taymans@chello.be>"
 };
 
 #define CLASS(o)	GST_MPEG_PARSE_CLASS (G_OBJECT_GET_CLASS (o))
@@ -54,39 +56,35 @@ enum {
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (sink_factory,
+static GstStaticPadTemplate sink_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "mpeg_parse_sink",
-    "video/mpeg",
-      "mpegversion",  GST_PROPS_INT_RANGE (1, 2),
-      "systemstream", GST_PROPS_BOOLEAN (TRUE)
+  GST_STATIC_CAPS ("video/mpeg, "
+      "mpegversion = (int) [ 1, 2 ], "
+      "systemstream = (boolean) TRUE"
   )
 );
 
-GST_PAD_TEMPLATE_FACTORY (src_factory,
+static GstStaticPadTemplate src_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "mpeg_parse_src",
-    "video/mpeg",
-      "mpegversion",  GST_PROPS_INT_RANGE (1, 2),
-      "systemstream", GST_PROPS_BOOLEAN (TRUE),
-      "parsed",       GST_PROPS_BOOLEAN (TRUE)
+  GST_STATIC_CAPS ("video/mpeg, "
+      "mpegversion = (int) [ 1, 2 ], "
+      "systemstream = (boolean) TRUE"
   )
 );
 
 static void 		gst_mpeg_parse_class_init	(GstMPEGParseClass *klass);
+static void 		gst_mpeg_parse_base_init	(GstMPEGParseClass *klass);
 static void 		gst_mpeg_parse_init		(GstMPEGParse *mpeg_parse);
 static GstElementStateReturn
 			gst_mpeg_parse_change_state	(GstElement *element);
 
 static void 		gst_mpeg_parse_set_clock 	(GstElement *element, GstClock *clock);
-static GstClock* 	gst_mpeg_parse_get_clock 	(GstElement *element);
-static GstClockTime	gst_mpeg_parse_get_time 	(GstClock *clock, gpointer data);
 
 static gboolean		gst_mpeg_parse_parse_packhead 	(GstMPEGParse *mpeg_parse, GstBuffer *buffer);
 static void 		gst_mpeg_parse_send_data	(GstMPEGParse *mpeg_parse, GstData *data, GstClockTime time);
@@ -113,7 +111,8 @@ gst_mpeg_parse_get_type (void)
 
   if (!mpeg_parse_type) {
     static const GTypeInfo mpeg_parse_info = {
-      sizeof(GstMPEGParseClass),      NULL,
+      sizeof(GstMPEGParseClass),
+      (GBaseInitFunc)gst_mpeg_parse_base_init,
       NULL,
       (GClassInitFunc)gst_mpeg_parse_class_init,
       NULL,
@@ -127,6 +126,14 @@ gst_mpeg_parse_get_type (void)
   return mpeg_parse_type;
 }
 
+static void
+gst_mpeg_parse_base_init (GstMPEGParseClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_set_details (element_class, &mpeg_parse_details);
+}
+
 static void
 gst_mpeg_parse_class_init (GstMPEGParseClass *klass) 
 {
@@ -144,15 +151,11 @@ gst_mpeg_parse_class_init (GstMPEGParseClass *klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAX_DISCONT,
     g_param_spec_int ("max_discont", "Max Discont", "The maximun allowed SCR discontinuity",
                       0, G_MAXINT, DEFAULT_MAX_DISCONT, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class, ARG_STREAMINFO,
-    g_param_spec_boxed ("streaminfo", "Streaminfo", "Streaminfo",
-                        GST_TYPE_CAPS, G_PARAM_READABLE));
 
   gobject_class->get_property = gst_mpeg_parse_get_property;
   gobject_class->set_property = gst_mpeg_parse_set_property;
 
   gstelement_class->change_state  = gst_mpeg_parse_change_state;
-  gstelement_class->get_clock     = gst_mpeg_parse_get_clock;
   gstelement_class->set_clock     = gst_mpeg_parse_set_clock;
   gstelement_class->get_index     = gst_mpeg_parse_get_index;
   gstelement_class->set_index     = gst_mpeg_parse_set_index;
@@ -164,19 +167,27 @@ gst_mpeg_parse_class_init (GstMPEGParseClass *klass)
   klass->parse_pes 	= NULL;
   klass->send_data 	= gst_mpeg_parse_send_data;
   klass->handle_discont	= gst_mpeg_parse_handle_discont;
+
+  /* FIXME: this is a hack.  We add the pad templates here instead
+   * in the base_init function, since the derived class (mpegdemux)
+   * uses different pads.  IMO, this is wrong. */
+  gst_element_class_add_pad_template (gstelement_class,
+		gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+		gst_static_pad_template_get (&sink_factory));
 }
 
 static void
 gst_mpeg_parse_init (GstMPEGParse *mpeg_parse)
 {
   mpeg_parse->sinkpad = gst_pad_new_from_template(
-		  GST_PAD_TEMPLATE_GET (sink_factory), "sink");
+		  gst_static_pad_template_get (&sink_factory), "sink");
   gst_element_add_pad(GST_ELEMENT(mpeg_parse),mpeg_parse->sinkpad);
   gst_pad_set_formats_function (mpeg_parse->sinkpad, gst_mpeg_parse_get_src_formats);
   gst_pad_set_convert_function (mpeg_parse->sinkpad, gst_mpeg_parse_convert_src);
 
   mpeg_parse->srcpad = gst_pad_new_from_template(
-		  GST_PAD_TEMPLATE_GET (src_factory), "src");
+		  gst_static_pad_template_get (&src_factory), "src");
   gst_element_add_pad(GST_ELEMENT(mpeg_parse),mpeg_parse->srcpad);
   gst_pad_set_formats_function (mpeg_parse->srcpad, gst_mpeg_parse_get_src_formats);
   gst_pad_set_convert_function (mpeg_parse->srcpad, gst_mpeg_parse_convert_src);
@@ -184,6 +195,7 @@ gst_mpeg_parse_init (GstMPEGParse *mpeg_parse)
   gst_pad_set_event_function (mpeg_parse->srcpad, gst_mpeg_parse_handle_src_event);
   gst_pad_set_query_type_function (mpeg_parse->srcpad, gst_mpeg_parse_get_src_query_types);
   gst_pad_set_query_function (mpeg_parse->srcpad, gst_mpeg_parse_handle_src_query);
+  gst_pad_use_explicit_caps (mpeg_parse->srcpad);
 
   gst_element_set_loop_function (GST_ELEMENT (mpeg_parse), gst_mpeg_parse_loop);
 
@@ -191,22 +203,10 @@ gst_mpeg_parse_init (GstMPEGParse *mpeg_parse)
   mpeg_parse->sync = FALSE;
   mpeg_parse->id = NULL;
   mpeg_parse->max_discont = DEFAULT_MAX_DISCONT;
-  mpeg_parse->provided_clock = gst_mpeg_clock_new ("MPEGParseClock", 
-		  gst_mpeg_parse_get_time, mpeg_parse);
-  mpeg_parse->streaminfo = NULL;
 
   GST_FLAG_SET (mpeg_parse, GST_ELEMENT_EVENT_AWARE);
 }
 
-static GstClock*
-gst_mpeg_parse_get_clock (GstElement *element)
-{   
-  /* GstMPEGParse *parse = GST_MPEG_PARSE (element); */
-
-  /* return parse->provided_clock; */
-  return NULL;
-}
-
 static void
 gst_mpeg_parse_set_clock (GstElement *element, GstClock *clock)
 {   
@@ -215,38 +215,31 @@ gst_mpeg_parse_set_clock (GstElement *element, GstClock *clock)
   parse->clock = clock;
 } 
 
-static GstClockTime
-gst_mpeg_parse_get_time (GstClock *clock, gpointer data)
-{   
-  GstMPEGParse *parse = GST_MPEG_PARSE (data);
-
-  return MPEGTIME_TO_GSTTIME (parse->current_scr);
-}
-
+#if 0
 static void
 gst_mpeg_parse_update_streaminfo (GstMPEGParse *mpeg_parse)
 {
   GstProps *props;
   GstPropsEntry *entry;
   gboolean mpeg2 = GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize);
+  GstCaps *caps;
 
   props = gst_props_empty_new ();
 
-  entry = gst_props_entry_new ("mpegversion", GST_PROPS_INT (mpeg2 ? 2 : 1));
+  entry = gst_props_entry_new ("mpegversion", G_TYPE_INT (mpeg2 ? 2 : 1));
   gst_props_add_entry (props, (GstPropsEntry *) entry);
 
-  entry = gst_props_entry_new ("bitrate", GST_PROPS_INT (mpeg_parse->mux_rate * 400)); 
+  entry = gst_props_entry_new ("bitrate", G_TYPE_INT (mpeg_parse->mux_rate * 400)); 
   gst_props_add_entry (props, (GstPropsEntry *) entry);
 
-  if (mpeg_parse->streaminfo) 
-    gst_caps_unref (mpeg_parse->streaminfo);
-
-  mpeg_parse->streaminfo = gst_caps_new ("mpeg_streaminfo",
-		                         "application/x-gst-streaminfo",
-					  props);
+  caps = gst_caps_new ("mpeg_streaminfo",
+	               "application/x-gst-streaminfo",
+	  	       props);
 
+  gst_caps_replace_sink (&mpeg_parse->streaminfo, caps);
   g_object_notify (G_OBJECT (mpeg_parse), "streaminfo");
 }
+#endif
 
 static void
 gst_mpeg_parse_send_data (GstMPEGParse *mpeg_parse, GstData *data, GstClockTime time)
@@ -264,25 +257,18 @@ gst_mpeg_parse_send_data (GstMPEGParse *mpeg_parse, GstData *data, GstClockTime
     if (!GST_PAD_CAPS (mpeg_parse->srcpad)) {
       gboolean mpeg2 = GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize);
 
-      if (gst_pad_try_set_caps (mpeg_parse->srcpad,
-		      GST_CAPS_NEW (
-    			"mpeg_parse_src",
-    			"video/mpeg",
-    			  "mpegversion",  GST_PROPS_INT (mpeg2 ? 2 : 1),
-    			  "systemstream", GST_PROPS_BOOLEAN (TRUE),
-    			  "parsed",       GST_PROPS_BOOLEAN (TRUE)
-			      )) < 0)
-      {
-	gst_element_error (GST_ELEMENT (mpeg_parse), "could no set source caps");
-	return;
-      }
+      gst_pad_set_explicit_caps (mpeg_parse->srcpad,
+	  gst_caps_new_simple ("video/mpeg",
+	    "mpegversion",  G_TYPE_INT,  (mpeg2 ? 2 : 1),
+	    "systemstream", G_TYPE_BOOLEAN, TRUE,
+	    "parsed",       G_TYPE_BOOLEAN, TRUE, NULL));
     }
 
     GST_BUFFER_TIMESTAMP (data) = time;
-    GST_DEBUG (0, "current_scr %lld", time);
+    GST_DEBUG ("current_scr %" G_GINT64_FORMAT, time);
 
     if (GST_PAD_IS_USABLE (mpeg_parse->srcpad))
-      gst_pad_push (mpeg_parse->srcpad, GST_BUFFER (data));
+      gst_pad_push (mpeg_parse->srcpad, GST_DATA (data));
     else
       gst_data_unref (data);
   }
@@ -297,7 +283,7 @@ gst_mpeg_parse_handle_discont (GstMPEGParse *mpeg_parse)
 		  MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr), NULL);
 
   if (GST_PAD_IS_USABLE (mpeg_parse->srcpad))
-    gst_pad_push (mpeg_parse->srcpad, GST_BUFFER (event));
+    gst_pad_push (mpeg_parse->srcpad, GST_DATA (event));
   else
     gst_event_unref (event);
 }
@@ -310,7 +296,7 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   guint32 scr1, scr2;
   guint32 new_rate;
 
-  GST_DEBUG (0, "in parse_packhead");
+  GST_DEBUG ("in parse_packhead");
 
   buf = GST_BUFFER_DATA (buffer);
   buf += 4;
@@ -322,16 +308,16 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
     guint32 scr_ext;
 
     /* :2=01 ! scr:3 ! marker:1==1 ! scr:15 ! marker:1==1 ! scr:15 */
-    scr  = (scr1 & 0x38000000) << 3;
-    scr |= (scr1 & 0x03fff800) << 4;
-    scr |= (scr1 & 0x000003ff) << 5;
-    scr |= (scr2 & 0xf8000000) >> 27;
+    scr  = ((guint64) scr1 & 0x38000000) << 3;
+    scr |= ((guint64) scr1 & 0x03fff800) << 4;
+    scr |= ((guint64) scr1 & 0x000003ff) << 5;
+    scr |= ((guint64) scr2 & 0xf8000000) >> 27;
 
     scr_ext = (scr2 & 0x03fe0000) >> 17;
 
     scr = (scr * 300 + scr_ext % 300) / 300;
 
-    GST_DEBUG (0, "%lld %d, %08x %08x %lld diff: %lld", 
+    GST_DEBUG ("%" G_GINT64_FORMAT " %d, %08x %08x %" G_GINT64_FORMAT " diff: %" G_GINT64_FORMAT, 
 		    scr, scr_ext, scr1, scr2, mpeg_parse->bytes_since_scr, 
 		    scr - mpeg_parse->current_scr);
 
@@ -339,17 +325,17 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
     new_rate = (GUINT32_FROM_BE ((*(guint32 *) buf)) & 0xfffffc00) >> 10;
   }
   else {
-    scr  = (scr1 & 0x0e000000) << 5;
-    scr |= (scr1 & 0x00fffe00) << 6;
-    scr |= (scr1 & 0x000000ff) << 7;
-    scr |= (scr2 & 0xfe000000) >> 25;
+    scr  = ((guint64) scr1 & 0x0e000000) << 5;
+    scr |= ((guint64) scr1 & 0x00fffe00) << 6;
+    scr |= ((guint64) scr1 & 0x000000ff) << 7;
+    scr |= ((guint64) scr2 & 0xfe000000) >> 25;
 
     buf += 5;
     /* we do this byte by byte because buf[3] might be outside of buf's
      * memory space */
-    new_rate = (buf[0] & 0x7f) << 15;
-    new_rate |= buf[1] << 7;
-    new_rate |= buf[2] >> 1;
+    new_rate  = ((gint32) buf[0] & 0x7f) << 15;
+    new_rate |= ((gint32) buf[1]) << 7;
+    new_rate |=           buf[2]  >> 1;
   }
 
   scr_orig = scr;
@@ -360,7 +346,9 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
     mpeg_parse->next_scr = scr;
   }
 
-  GST_DEBUG (0, "SCR is %llu (%llu) next: %lld (%lld) diff: %lld (%lld)", 
+  GST_DEBUG ("SCR is %" G_GUINT64_FORMAT " (%" G_GUINT64_FORMAT ") next: %" 
+	     G_GINT64_FORMAT " (%" G_GINT64_FORMAT ") diff: %" G_GINT64_FORMAT " (%" 
+	     G_GINT64_FORMAT ")", 
 		  scr, 
 		  MPEGTIME_TO_GSTTIME (scr),
 		  mpeg_parse->next_scr,
@@ -370,13 +358,15 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
 		  MPEGTIME_TO_GSTTIME (mpeg_parse->next_scr));
 
   if (ABS ((gint64)mpeg_parse->next_scr - (gint64)(scr_adj)) > mpeg_parse->max_discont) {
-    GST_DEBUG (0, "discontinuity detected; expected: %llu got: %llu real:%lld adjust:%lld", 
+    GST_DEBUG ("discontinuity detected; expected: %" 
+	       G_GUINT64_FORMAT " got: %" G_GUINT64_FORMAT " real:%" 
+	       G_GINT64_FORMAT " adjust:%" G_GINT64_FORMAT, 
            mpeg_parse->next_scr, scr_adj, scr, mpeg_parse->adjust);
 
     mpeg_parse->adjust = mpeg_parse->next_scr - scr;
     scr = mpeg_parse->next_scr;
 
-    GST_DEBUG (0, "new adjust: %lld", mpeg_parse->adjust);
+    GST_DEBUG ("new adjust: %" G_GINT64_FORMAT, mpeg_parse->adjust);
   }
   else {
     scr = scr_adj;
@@ -396,8 +386,8 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
   if (mpeg_parse->mux_rate != new_rate) {
     mpeg_parse->mux_rate = new_rate;
 
-    gst_mpeg_parse_update_streaminfo (mpeg_parse);
-    GST_DEBUG (0, "stream is %1.3fMbs", (mpeg_parse->mux_rate * 400) / 1000000.0);
+    //gst_mpeg_parse_update_streaminfo (mpeg_parse);
+    GST_DEBUG ("stream is %1.3fMbs", (mpeg_parse->mux_rate * 400) / 1000000.0);
   }
 
   return TRUE;
@@ -412,15 +402,9 @@ gst_mpeg_parse_loop (GstElement *element)
   gboolean mpeg2;
   GstClockTime time;
 
-  /* maybe later we can make this less fatal, but right now this shouldn't
-   * happen at all */
-  g_assert (GST_IS_ELEMENT (element));
-
   data = gst_mpeg_packetize_read (mpeg_parse->packetize);
-  if (data == NULL) {
-    g_warning ("gst_mpeg_packetize_read returned a NULL buffer, not good !");
+  if (!data)
     return;
-  }
 
   id = GST_MPEG_PACKETIZE_ID (mpeg_parse->packetize);
   mpeg2 = GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize);
@@ -428,9 +412,11 @@ gst_mpeg_parse_loop (GstElement *element)
   if (GST_IS_BUFFER (data)) {
     GstBuffer *buffer = GST_BUFFER (data);
 
-    GST_DEBUG (0, "have chunk 0x%02X", id);
+    GST_DEBUG ("have chunk 0x%02X", id);
 
     switch (id) {
+      case 0xb9:
+	break;
       case 0xba:
         if (CLASS (mpeg_parse)->parse_packhead) {
 	  CLASS (mpeg_parse)->parse_packhead (mpeg_parse, buffer);
@@ -467,7 +453,7 @@ gst_mpeg_parse_loop (GstElement *element)
 
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_DISCONTINUOUS:
-        GST_DEBUG (GST_CAT_EVENT, "event: %d\n", GST_EVENT_TYPE (data));
+        GST_DEBUG ("event: %d\n", GST_EVENT_TYPE (data));
 
         mpeg_parse->discont_pending = TRUE;
         mpeg_parse->packetize->resync = TRUE;
@@ -488,7 +474,7 @@ gst_mpeg_parse_loop (GstElement *element)
     if (mpeg_parse->discont_pending) {
       if (!mpeg_parse->scr_pending) {
         if (mpeg_parse->clock && mpeg_parse->sync) {
-          gst_clock_handle_discont (mpeg_parse->clock, MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr));
+          gst_element_set_time (GST_ELEMENT (mpeg_parse), MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr));
         }
 	if (CLASS (mpeg_parse)->handle_discont) {
 	  CLASS (mpeg_parse)->handle_discont (mpeg_parse);
@@ -496,11 +482,10 @@ gst_mpeg_parse_loop (GstElement *element)
         mpeg_parse->discont_pending = FALSE;
       }
       else {
-	GST_DEBUG (0, "waiting for SCR\n");
+	GST_DEBUG ("waiting for SCR");
+        gst_buffer_unref (GST_BUFFER (data));
+        return;
       }
-      gst_buffer_unref (GST_BUFFER (data));
-      return;
-
     }
 
     size = GST_BUFFER_SIZE (data);
@@ -510,15 +495,12 @@ gst_mpeg_parse_loop (GstElement *element)
       gboolean mpeg2 = GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize);
 
       if (gst_pad_try_set_caps (mpeg_parse->sinkpad,
-		      GST_CAPS_NEW (
-    			"mpeg_parse_src",
-    			"video/mpeg",
-    			  "mpegversion",  GST_PROPS_INT (mpeg2 ? 2 : 1),
-    			  "systemstream", GST_PROPS_BOOLEAN (TRUE),
-    			  "parsed",       GST_PROPS_BOOLEAN (TRUE)
-			      )) < 0)
+	    gst_caps_new_simple ("video/mpeg",
+	      "mpegversion",  G_TYPE_INT,  (mpeg2 ? 2 : 1),
+	      "systemstream", G_TYPE_BOOLEAN, TRUE,
+	      "parsed",       G_TYPE_BOOLEAN, TRUE, NULL)) < 0)
       {
-	gst_element_error (GST_ELEMENT (mpeg_parse), "could no set sink caps");
+	GST_ELEMENT_ERROR (mpeg_parse, CORE, NEGOTIATION, (NULL), (NULL));
 	return;
       }
     }
@@ -527,12 +509,8 @@ gst_mpeg_parse_loop (GstElement *element)
       CLASS (mpeg_parse)->send_data (mpeg_parse, data, time);
 
     if (mpeg_parse->clock && mpeg_parse->sync && !mpeg_parse->discont_pending) {
-      mpeg_parse->id = gst_clock_new_single_shot_id (mpeg_parse->clock, time);
-
-      GST_DEBUG (GST_CAT_CLOCK, "syncing mpegparse");
-      gst_element_clock_wait (GST_ELEMENT (mpeg_parse), mpeg_parse->id, NULL);
-      gst_clock_id_free (mpeg_parse->id);
-      mpeg_parse->id = NULL;
+      GST_DEBUG ("syncing mpegparse");
+      gst_element_wait (GST_ELEMENT (mpeg_parse), time);
     }
 
     if (mpeg_parse->current_scr != -1)
@@ -562,7 +540,7 @@ gst_mpeg_parse_loop (GstElement *element)
         mpeg_parse->next_scr = scr;
       }
 
-      GST_DEBUG (0, "size: %lld, total since SCR: %lld, next SCR: %lld", 
+      GST_DEBUG ("size: %" G_GINT64_FORMAT ", total since SCR: %" G_GINT64_FORMAT ", next SCR: %" G_GINT64_FORMAT, 
 		       size, bss, mpeg_parse->next_scr);
     }
   }
@@ -711,6 +689,12 @@ index_seek (GstPad *pad, GstEvent *event, guint64 *offset, gint64 *scr)
     if (gst_index_entry_assoc_map (entry, GST_FORMAT_TIME, &time)) {
       *scr = GSTTIME_TO_MPEGTIME (time);
     }
+    GST_CAT_DEBUG (GST_CAT_SEEK, "%s:%s index %s %" G_GINT64_FORMAT
+		   " -> %" G_GINT64_FORMAT " bytes, scr=%" G_GINT64_FORMAT,
+		   GST_DEBUG_PAD_NAME (pad),
+		   gst_format_get_details (GST_EVENT_SEEK_FORMAT (event))->nick,
+		   GST_EVENT_SEEK_OFFSET (event),
+		   *offset, *scr);
     return TRUE;
   }
   return FALSE;
@@ -766,7 +750,7 @@ gst_mpeg_parse_handle_src_event (GstPad *pad, GstEvent *event)
       if (!res)
 	break;
 	      
-      GST_DEBUG (0, "sending seek to %lld", desired_offset);
+      GST_DEBUG ("sending seek to %" G_GINT64_FORMAT, desired_offset);
       if (gst_bytestream_seek (mpeg_parse->packetize->bs, desired_offset, GST_SEEK_METHOD_SET)) {
         mpeg_parse->discont_pending = TRUE;
         mpeg_parse->scr_pending = TRUE;
@@ -810,6 +794,7 @@ gst_mpeg_parse_change_state (GstElement *element)
         gst_mpeg_packetize_destroy (mpeg_parse->packetize);
         mpeg_parse->packetize = NULL;
       }
+      //gst_caps_replace (&mpeg_parse->streaminfo, NULL);
       break;
     default:
       break;
@@ -832,9 +817,6 @@ gst_mpeg_parse_get_property (GObject *object, guint prop_id, GValue *value, GPar
     case ARG_MAX_DISCONT: 
       g_value_set_int (value, mpeg_parse->max_discont);
       break;
-    case ARG_STREAMINFO: 
-      g_value_set_boxed (value, mpeg_parse->streaminfo);
-      break;
     default: 
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -894,32 +876,18 @@ gst_mpeg_parse_release_locks (GstElement *element)
   mpeg_parse = GST_MPEG_PARSE (element);
 
   if (mpeg_parse->id) {
-    gst_clock_id_unlock (mpeg_parse->id);
+    /* FIXME */
+    //gst_clock_id_unlock (mpeg_parse->id);
   }
 
   return TRUE;
 }
 
 gboolean
-gst_mpeg_parse_plugin_init (GModule *module, GstPlugin *plugin)
+gst_mpeg_parse_plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* this filter needs the bytestream package */
-  if (!gst_library_load ("gstbytestream"))
-    return FALSE;
-
-  /* create an elementfactory for the mpeg_parse element */
-  factory = gst_element_factory_new ("mpegparse", GST_TYPE_MPEG_PARSE,
-                                     &mpeg_parse_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-
   scr_format = gst_format_register ("scr", "The MPEG system clock reference time");
 
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (sink_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "mpegparse",
+			       GST_RANK_NONE, GST_TYPE_MPEG_PARSE);
 }
