@@ -67,76 +67,115 @@ static void gst_videoscale_scale_nearest_24bit (GstVideoscale *scale,
 static void gst_videoscale_scale_nearest_16bit (GstVideoscale *scale,
 	unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh);
 
+#define fourcc_YUY2 GST_MAKE_FOURCC('Y','U','Y','2')
+#define fourcc_UYVY GST_MAKE_FOURCC('U','Y','V','Y')
+#define fourcc_Y422 GST_MAKE_FOURCC('Y','4','2','2')
+#define fourcc_UYNV GST_MAKE_FOURCC('U','Y','N','V')
+#define fourcc_YVYU GST_MAKE_FOURCC('Y','V','Y','U')
+#define fourcc_YV12 GST_MAKE_FOURCC('Y','V','1','2')
+#define fourcc_I420 GST_MAKE_FOURCC('I','4','2','0')
+#define fourcc_Y800 GST_MAKE_FOURCC('Y','8','0','0')
+#define fourcc_RGB_ GST_MAKE_FOURCC('R','G','B',' ')
+
 struct videoscale_format_struct videoscale_formats[] = {
 	/* packed */
-	{ "YUY2", 16, gst_videoscale_packed422, },
-	{ "UYVY", 16, gst_videoscale_packed422rev, },
-	{ "Y422", 16, gst_videoscale_packed422rev, },
-	{ "UYNV", 16, gst_videoscale_packed422rev, },
-	{ "YVYU", 16, gst_videoscale_packed422, },
+	{ fourcc_YUY2, 16, gst_videoscale_packed422, },
+	{ fourcc_UYVY, 16, gst_videoscale_packed422rev, },
+	{ fourcc_Y422, 16, gst_videoscale_packed422rev, },
+	{ fourcc_UYNV, 16, gst_videoscale_packed422rev, },
+	{ fourcc_YVYU, 16, gst_videoscale_packed422, },
 	/* planar */
-	{ "YV12", 12, gst_videoscale_planar411, },
-	{ "I420", 12, gst_videoscale_planar411, },
-	{ "Y800", 8,  gst_videoscale_planar400, },
+	{ fourcc_YV12, 12, gst_videoscale_planar411, },
+	{ fourcc_I420, 12, gst_videoscale_planar411, },
+	{ fourcc_Y800, 8,  gst_videoscale_planar400, },
 	/* RGB */
-	{ "RGB ", 32, gst_videoscale_32bit, 24, G_BIG_ENDIAN, 0x00ff0000, 0x0000ff00, 0x000000ff },
-	{ "RGB ", 32, gst_videoscale_32bit, 24, G_BIG_ENDIAN, 0x000000ff, 0x0000ff00, 0x00ff0000 },
-	{ "RGB ", 32, gst_videoscale_32bit, 24, G_BIG_ENDIAN, 0xff000000, 0x00ff0000, 0x0000ff00 },
-	{ "RGB ", 32, gst_videoscale_32bit, 24, G_BIG_ENDIAN, 0x0000ff00, 0x00ff0000, 0xff000000 },
-	{ "RGB ", 24, gst_videoscale_24bit, 24, G_BIG_ENDIAN, 0xff0000, 0x00ff00, 0x0000ff },
-	{ "RGB ", 24, gst_videoscale_24bit, 24, G_BIG_ENDIAN, 0x0000ff, 0x00ff00, 0xff0000 },
-	{ "RGB ", 16, gst_videoscale_16bit, 16, G_BYTE_ORDER, 0xf800, 0x07e0, 0x001f },
-	{ "RGB ", 16, gst_videoscale_16bit, 15, G_BYTE_ORDER, 0x7c00, 0x03e0, 0x001f },
+	{ fourcc_RGB_, 32, gst_videoscale_32bit, 24, G_BIG_ENDIAN, 0x00ff0000, 0x0000ff00, 0x000000ff },
+	{ fourcc_RGB_, 32, gst_videoscale_32bit, 24, G_BIG_ENDIAN, 0x000000ff, 0x0000ff00, 0x00ff0000 },
+	{ fourcc_RGB_, 32, gst_videoscale_32bit, 24, G_BIG_ENDIAN, 0xff000000, 0x00ff0000, 0x0000ff00 },
+	{ fourcc_RGB_, 32, gst_videoscale_32bit, 24, G_BIG_ENDIAN, 0x0000ff00, 0x00ff0000, 0xff000000 },
+	{ fourcc_RGB_, 24, gst_videoscale_24bit, 24, G_BIG_ENDIAN, 0xff0000, 0x00ff00, 0x0000ff },
+	{ fourcc_RGB_, 24, gst_videoscale_24bit, 24, G_BIG_ENDIAN, 0x0000ff, 0x00ff00, 0xff0000 },
+	{ fourcc_RGB_, 16, gst_videoscale_16bit, 16, G_BYTE_ORDER, 0xf800, 0x07e0, 0x001f },
+	{ fourcc_RGB_, 16, gst_videoscale_16bit, 15, G_BYTE_ORDER, 0x7c00, 0x03e0, 0x001f },
 };
 
 int videoscale_n_formats = sizeof(videoscale_formats)/sizeof(videoscale_formats[0]);
 
-GstCaps *
-videoscale_get_caps(struct videoscale_format_struct *format)
+GstStructure *
+videoscale_get_structure (struct videoscale_format_struct *format)
 {
-  unsigned int fourcc;
-  GstCaps *caps;
+  GstStructure *structure;
 
   if(format->scale==NULL)
     return NULL;
 
-  fourcc = GST_MAKE_FOURCC(format->fourcc[0],format->fourcc[1],format->fourcc[2],format->fourcc[3]);
-
-  if(format->bpp){
-    caps = GST_CAPS_NEW ("videoscale", "video/x-raw-rgb",
-		"depth", GST_PROPS_INT(format->bpp),
-		"bpp", GST_PROPS_INT(format->depth),
-		"endianness", GST_PROPS_INT(format->endianness),
-		"red_mask", GST_PROPS_INT(format->red_mask),
-		"green_mask", GST_PROPS_INT(format->green_mask),
-		"blue_mask", GST_PROPS_INT(format->blue_mask));
+  if(format->depth){
+    structure = gst_structure_new ("video/x-raw-rgb",
+	"depth", G_TYPE_INT, format->depth,
+	"bpp", G_TYPE_INT, format->bpp,
+	"endianness", G_TYPE_INT, format->endianness,
+	"red_mask", G_TYPE_INT, format->red_mask,
+	"green_mask", G_TYPE_INT, format->green_mask,
+	"blue_mask", G_TYPE_INT, format->blue_mask,
+	NULL);
   }else{
-    caps = GST_CAPS_NEW ("videoscale", "video/x-raw-yuv",
-		"format", GST_PROPS_FOURCC (fourcc));
+    structure = gst_structure_new ("video/x-raw-yuv",
+	"format", GST_TYPE_FOURCC, format->fourcc, NULL);
   }
 
-  return caps;
+  gst_structure_set(structure,
+      "width", GST_TYPE_INT_RANGE, 100, G_MAXINT,
+      "height", GST_TYPE_INT_RANGE, 100, G_MAXINT,
+      "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE,
+      NULL);
+
+  return structure;
 }
 
 struct videoscale_format_struct *
-videoscale_find_by_caps(GstCaps *caps)
+videoscale_find_by_structure(GstStructure *structure)
 {
   int i;
+  gboolean ret;
+  struct videoscale_format_struct *format;
 
-  GST_DEBUG ("finding %p",caps);
+  GST_DEBUG ("finding %s",gst_structure_to_string(structure));
 
-  g_return_val_if_fail(caps != NULL, NULL);
+  g_return_val_if_fail(structure != NULL, NULL);
 
-  for (i = 0; i < videoscale_n_formats; i++){
-    GstCaps *c;
+  if(strcmp(gst_structure_get_name(structure),"video/x-raw-yuv")==0){
+    unsigned int fourcc;
 
-    c = videoscale_get_caps(videoscale_formats + i);
-    if(c){
-      if(gst_caps_is_always_compatible(caps, c)){
-        gst_caps_unref(c);
-        return videoscale_formats + i;
+    ret = gst_structure_get_fourcc (structure, "format", &fourcc);
+    if(!ret) return NULL;
+    for (i = 0; i < videoscale_n_formats; i++){
+      format = videoscale_formats + i;
+      if(format->depth==0 && format->fourcc == fourcc){
+	return format;
+      }
+    }
+  }else{
+    int bpp;
+    int depth;
+    int endianness;
+    int red_mask;
+    int green_mask;
+    int blue_mask;
+
+    ret = gst_structure_get_int (structure, "bpp", &bpp);
+    ret &= gst_structure_get_int (structure, "depth", &depth);
+    ret &= gst_structure_get_int (structure, "endianness", &endianness);
+    ret &= gst_structure_get_int (structure, "red_mask", &red_mask);
+    ret &= gst_structure_get_int (structure, "green_mask", &green_mask);
+    ret &= gst_structure_get_int (structure, "blue_mask", &blue_mask);
+    if(!ret) return NULL;
+    for (i = 0; i < videoscale_n_formats; i++){
+      format = videoscale_formats + i;
+      if(format->bpp == bpp && format->depth == depth &&
+	  format->endianness == endianness && format->red_mask == red_mask &&
+	  format->green_mask == green_mask && format->blue_mask == blue_mask) {
+	return format;
       }
-      gst_caps_unref(c);
     }
   }
 
@@ -149,30 +188,35 @@ gst_videoscale_setup (GstVideoscale *videoscale)
   g_return_if_fail (GST_IS_VIDEOSCALE (videoscale));
   g_return_if_fail (videoscale->format != NULL);
 
-  GST_DEBUG ("format=%p \"%s\" from %dx%d to %dx%d",
-		videoscale->format, videoscale->format->fourcc,
-		videoscale->from_width, videoscale->from_height,
-		videoscale->to_width, videoscale->to_height);
+  GST_DEBUG_OBJECT (videoscale, "format=%p " GST_FOURCC_FORMAT
+                    " from %dx%d to %dx%d",
+		    videoscale->format,
+                    GST_FOURCC_ARGS(videoscale->format->fourcc),
+		    videoscale->from_width, videoscale->from_height,
+		    videoscale->to_width, videoscale->to_height);
 
   if(videoscale->to_width==0 || videoscale->to_height==0 ||
   	videoscale->from_width==0 || videoscale->from_height==0){
+    g_critical ("bad sizes %dx%d %dx%d",
+  	videoscale->from_width, videoscale->from_height,
+        videoscale->to_width, videoscale->to_height);
     return;
   }
 
   if(videoscale->to_width == videoscale->from_width &&
 	videoscale->to_height == videoscale->from_height){
-    GST_DEBUG ("videoscale: using passthru");
+    GST_DEBUG_OBJECT (videoscale, "using passthru");
     videoscale->passthru = TRUE;
     videoscale->inited = TRUE;
     return;
   }
 
-  GST_DEBUG ("videoscale: scaling method POINT_SAMPLE");
+  GST_DEBUG_OBJECT (videoscale, "scaling method POINT_SAMPLE");
 
   videoscale->from_buf_size = (videoscale->from_width * videoscale->from_height
-		  * videoscale->format->depth) / 8;
+		  * videoscale->format->bpp) / 8;
   videoscale->to_buf_size = (videoscale->to_width * videoscale->to_height
-		  * videoscale->format->depth) / 8;
+		  * videoscale->format->bpp) / 8;
 
   videoscale->passthru = FALSE;
   videoscale->inited = TRUE;
@@ -186,7 +230,7 @@ gst_videoscale_scale_rgb (GstVideoscale *scale, unsigned char *dest, unsigned ch
   int sh = scale->from_height;
   int dw = scale->to_width;
   int dh = scale->to_height;
-  GST_DEBUG ("videoscale: scaling RGB %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_DEBUG_OBJECT (scale, "scaling RGB %dx%d to %dx%d", sw, sh, dw, dh);
 
   switch (scale->scale_bytes) {
     case 2:
@@ -201,7 +245,7 @@ gst_videoscale_scale_rgb (GstVideoscale *scale, unsigned char *dest, unsigned ch
      break;
   }
 
-  GST_DEBUG ("videoscale: %p %p", src, dest);
+  GST_DEBUG_OBJECT (scale, "%p %p", src, dest);
   //scale->scaler(scale, src, dest, sw, sh, dw, dh);
 }
 #endif
@@ -214,7 +258,7 @@ gst_videoscale_planar411 (GstVideoscale *scale, unsigned char *dest, unsigned ch
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG ("videoscale: scaling planar 4:1:1 %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_DEBUG_OBJECT (scale, "scaling planar 4:1:1 %dx%d to %dx%d", sw, sh, dw, dh);
 
   gst_videoscale_scale_nearest(scale, dest, src, sw, sh, dw, dh);
 
@@ -242,7 +286,7 @@ gst_videoscale_planar400 (GstVideoscale *scale, unsigned char *dest, unsigned ch
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG ("videoscale: scaling Y-only %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_DEBUG_OBJECT (scale, "scaling Y-only %dx%d to %dx%d", sw, sh, dw, dh);
 
   gst_videoscale_scale_nearest(scale, dest, src, sw, sh, dw, dh);
 }
@@ -255,7 +299,7 @@ gst_videoscale_packed422 (GstVideoscale *scale, unsigned char *dest, unsigned ch
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG ("videoscale: scaling 4:2:2 %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_DEBUG_OBJECT (scale, "scaling 4:2:2 %dx%d to %dx%d", sw, sh, dw, dh);
 
   gst_videoscale_scale_nearest_str2(scale, dest, src, sw, sh, dw, dh);
   gst_videoscale_scale_nearest_str4(scale, dest+1, src+1, sw/2, sh, dw/2, dh);
@@ -271,7 +315,7 @@ gst_videoscale_packed422rev (GstVideoscale *scale, unsigned char *dest, unsigned
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG ("videoscale: scaling 4:2:2 %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_DEBUG_OBJECT (scale, "scaling 4:2:2 %dx%d to %dx%d", sw, sh, dw, dh);
 
   gst_videoscale_scale_nearest_str2(scale, dest+1, src, sw, sh, dw, dh);
   gst_videoscale_scale_nearest_str4(scale, dest, src+1, sw/2, sh, dw/2, dh);
@@ -287,7 +331,7 @@ gst_videoscale_32bit (GstVideoscale *scale, unsigned char *dest, unsigned char *
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG ("videoscale: scaling 32bit %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_DEBUG_OBJECT (scale, "scaling 32bit %dx%d to %dx%d", sw, sh, dw, dh);
 
   gst_videoscale_scale_nearest_32bit(scale, dest, src, sw, sh, dw, dh);
 
@@ -301,7 +345,7 @@ gst_videoscale_24bit (GstVideoscale *scale, unsigned char *dest, unsigned char *
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG ("videoscale: scaling 24bit %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_DEBUG_OBJECT (scale, "scaling 24bit %dx%d to %dx%d", sw, sh, dw, dh);
 
   gst_videoscale_scale_nearest_24bit(scale, dest, src, sw, sh, dw, dh);
 
@@ -315,7 +359,7 @@ gst_videoscale_16bit (GstVideoscale *scale, unsigned char *dest, unsigned char *
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG ("videoscale: scaling 16bit %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_DEBUG_OBJECT (scale, "scaling 16bit %dx%d to %dx%d", sw, sh, dw, dh);
 
   gst_videoscale_scale_nearest_16bit(scale, dest, src, sw, sh, dw, dh);
 
@@ -334,7 +378,7 @@ gst_videoscale_bilinear (unsigned char *src, double x, double y, int sw, int sh)
   double dest;
   int color;
 
-  GST_DEBUG ("videoscale: scaling bilinear %f %f %dx%d", x, y, sw, sh);
+  GST_DEBUG_OBJECT (scale, "scaling bilinear %f %f %dx%d", x, y, sw, sh);
 
   dest=(1-a)*(1-b)*RC(j,k)+
        a*(1-b)*RC(j+1,k);
@@ -363,7 +407,7 @@ gst_videoscale_bicubic (unsigned char *src, double x, double y, int sw, int sh)
   double t1, t2, t3, t4;
   double a1, a2, a3, a4;
 
-  GST_DEBUG ("videoscale: scaling bicubic %dx%d", sw, sh);
+  GST_DEBUG_OBJECT (scale, "scaling bicubic %dx%d", sw, sh);
 
   a1 = -a*(1-a)*(1-a);
   a2 = (1-2*a*a+a*a*a);
@@ -396,17 +440,17 @@ gst_videoscale_scale_plane_slow (GstVideoscale *scale, unsigned char *src, unsig
   double xr, yr;
   int x, y;
 
-  GST_DEBUG ("videoscale: scale plane slow %dx%d %dx%d %g %g %p %p", sw, sh, dw, dh, zoomx, zoomy, src, dest);
+  GST_DEBUG_OBJECT (scale, "scale plane slow %dx%d %dx%d %g %g %p %p", sw, sh, dw, dh, zoomx, zoomy, src, dest);
 
   for (y=0; y<dh; y++) {
     yr = ((double)y)/zoomy;
     for (x=0; x<dw; x++) {
       xr = ((double)x)/zoomx;
 
-      GST_DEBUG ("videoscale: scale plane slow %g %g %p", xr, yr, (src+(int)(x)+(int)((y)*sw)));
+      GST_DEBUG_OBJECT (scale, "scale plane slow %g %g %p", xr, yr, (src+(int)(x)+(int)((y)*sw)));
 
       if (floor(xr) == xr && floor(yr) == yr){
-        GST_DEBUG ("videoscale: scale plane %g %g %p %p", xr, yr, (src+(int)(x)+(int)((y)*sw)), dest);
+        GST_DEBUG_OBJECT (scale, "scale plane %g %g %p %p", xr, yr, (src+(int)(x)+(int)((y)*sw)), dest);
 	*dest++ = RC(xr, yr);
       }
       else {
@@ -428,7 +472,7 @@ gst_videoscale_scale_point_sample (GstVideoscale *scale, unsigned char *src, uns
   int sum, xcount, ycount, loop;
   unsigned char *srcp, *srcp2;
 
-  GST_DEBUG ("videoscale: scaling nearest point sample %p %p %d", src, dest, dw);
+  GST_DEBUG_OBJECT (scale, "scaling nearest point sample %p %p %d", src, dest, dw);
 
   ypos = 0x10000;
   yinc = (sh<<16)/dh;
@@ -479,7 +523,7 @@ gst_videoscale_scale_nearest (GstVideoscale *scale,
   guchar *destp = dest;
   guchar *srcp = src;
 
-  GST_DEBUG ("videoscale: scaling nearest %p %p %d", src, dest, dw);
+  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
 
 
   ypos = 0x10000;
@@ -523,7 +567,7 @@ gst_videoscale_scale_nearest_str2 (GstVideoscale *scale,
   guchar *destp = dest;
   guchar *srcp = src;
 
-  GST_DEBUG ("videoscale: scaling nearest %p %p %d", src, dest, dw);
+  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
 
 
   ypos = 0x10000;
@@ -568,7 +612,7 @@ gst_videoscale_scale_nearest_str4 (GstVideoscale *scale,
   guchar *destp = dest;
   guchar *srcp = src;
 
-  GST_DEBUG ("videoscale: scaling nearest %p %p %d", src, dest, dw);
+  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
 
 
   ypos = 0x10000;
@@ -613,7 +657,7 @@ gst_videoscale_scale_nearest_32bit (GstVideoscale *scale,
   guchar *destp = dest;
   guchar *srcp = src;
 
-  GST_DEBUG ("videoscale: scaling nearest %p %p %d", src, dest, dw);
+  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
 
 
   ypos = 0x10000;
@@ -658,7 +702,7 @@ gst_videoscale_scale_nearest_24bit (GstVideoscale *scale,
   guchar *destp = dest;
   guchar *srcp = src;
 
-  GST_DEBUG ("videoscale: scaling nearest %p %p %d", src, dest, dw);
+  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
 
 
   ypos = 0x10000;
@@ -705,7 +749,7 @@ gst_videoscale_scale_nearest_16bit (GstVideoscale *scale,
   guchar *destp = dest;
   guchar *srcp = src;
 
-  GST_DEBUG ("videoscale: scaling nearest %p %p %d", src, dest, dw);
+  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
 
 
   ypos = 0x10000;
