@@ -28,17 +28,13 @@
 static GstElementDetails gst_v4lmjpegsink_details = {
   "Video (video4linux/MJPEG) sink",
   "Sink/Video",
-  "LGPL",
   "Writes MJPEG-encoded frames to a zoran MJPEG/video4linux device",
-  VERSION,
-  "Ronald Bultje <rbultje@ronald.bitfreak.net>",
-  "(C) 2001",
+  "Ronald Bultje <rbultje@ronald.bitfreak.net>"
 };
 
 /* v4lmjpegsink signals and args */
 enum {
   SIGNAL_FRAME_DISPLAYED,
-  SIGNAL_HAVE_SIZE,
   LAST_SIGNAL
 };
 
@@ -54,14 +50,15 @@ enum {
 
 
 /* init functions */
+static void		     gst_v4lmjpegsink_base_init   (gpointer g_class);
 static void                  gst_v4lmjpegsink_class_init   (GstV4lMjpegSinkClass *klass);
 static void                  gst_v4lmjpegsink_init         (GstV4lMjpegSink      *v4lmjpegsink);
 
 /* the chain of buffers */
 static GstPadLinkReturn   gst_v4lmjpegsink_sinkconnect  (GstPad               *pad,
-                                                            GstCaps              *vscapslist);
+                                                            const GstCaps              *vscapslist);
 static void                  gst_v4lmjpegsink_chain        (GstPad               *pad,
-                                                            GstBuffer            *buf);
+                                                            GstData              *_data);
 
 /* get/set gst object functions */
 static void                  gst_v4lmjpegsink_set_property (GObject              *object,
@@ -75,15 +72,6 @@ static void                  gst_v4lmjpegsink_get_property (GObject
 static GstElementStateReturn gst_v4lmjpegsink_change_state (GstElement           *element);
 static void		     gst_v4lmjpegsink_set_clock    (GstElement *element, GstClock *clock);
 
-/* bufferpool functions */
-static GstBuffer*            gst_v4lmjpegsink_buffer_new   (GstBufferPool  *pool,
-                                                            guint64        offset,
-                                                            guint          size,
-                                                            gpointer       user_data);
-
-
-static GstCaps *capslist = NULL;
-static GstPadTemplate *sink_template;
 
 static GstElementClass *parent_class = NULL;
 static guint gst_v4lmjpegsink_signals[LAST_SIGNAL] = { 0 };
@@ -97,7 +85,7 @@ gst_v4lmjpegsink_get_type (void)
   if (!v4lmjpegsink_type) {
     static const GTypeInfo v4lmjpegsink_info = {
       sizeof(GstV4lMjpegSinkClass),
-      NULL,
+      gst_v4lmjpegsink_base_init,
       NULL,
       (GClassInitFunc)gst_v4lmjpegsink_class_init,
       NULL,
@@ -111,7 +99,25 @@ gst_v4lmjpegsink_get_type (void)
   return v4lmjpegsink_type;
 }
 
+static void
+gst_v4lmjpegsink_base_init (gpointer g_class)
+{
+  static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE (
+      "sink",
+      GST_PAD_SINK,
+      GST_PAD_ALWAYS,
+      GST_STATIC_CAPS ("video/x-jpeg, "
+	"width = (int) [ 1, MAX ], "
+	"height = (int) [ 1, MAX ], "
+	"framerate = (double) [ 0, MAX ]")
+  );
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
+  gst_element_class_set_details (gstelement_class, &gst_v4lmjpegsink_details);
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get(&sink_template));
+}
 static void
 gst_v4lmjpegsink_class_init (GstV4lMjpegSinkClass *klass)
 {
@@ -151,12 +157,6 @@ gst_v4lmjpegsink_class_init (GstV4lMjpegSinkClass *klass)
     g_signal_new ("frame_displayed", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
                    G_STRUCT_OFFSET (GstV4lMjpegSinkClass, frame_displayed), NULL, NULL,
                    g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-  gst_v4lmjpegsink_signals[SIGNAL_HAVE_SIZE] =
-    g_signal_new ("have_size", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
-                   G_STRUCT_OFFSET (GstV4lMjpegSinkClass, have_size), NULL, NULL,
-                   gst_marshal_VOID__INT_INT, G_TYPE_NONE, 2,
-		   G_TYPE_UINT, G_TYPE_UINT);
-
 
   gstelement_class->change_state = gst_v4lmjpegsink_change_state;
   gstelement_class->set_clock    = gst_v4lmjpegsink_set_clock;
@@ -166,7 +166,10 @@ gst_v4lmjpegsink_class_init (GstV4lMjpegSinkClass *klass)
 static void
 gst_v4lmjpegsink_init (GstV4lMjpegSink *v4lmjpegsink)
 {
-  v4lmjpegsink->sinkpad = gst_pad_new_from_template (sink_template, "sink");
+  GstElementClass *klass = GST_ELEMENT_GET_CLASS (v4lmjpegsink);
+
+  v4lmjpegsink->sinkpad = gst_pad_new_from_template (
+	gst_element_class_get_pad_template (klass, "sink"), "sink");
   gst_element_add_pad (GST_ELEMENT (v4lmjpegsink), v4lmjpegsink->sinkpad);
 
   gst_pad_set_chain_function (v4lmjpegsink->sinkpad, gst_v4lmjpegsink_chain);
@@ -184,62 +187,44 @@ gst_v4lmjpegsink_init (GstV4lMjpegSink *v4lmjpegsink)
   v4lmjpegsink->bufsize = 256;
 
   GST_FLAG_SET(v4lmjpegsink, GST_ELEMENT_THREAD_SUGGESTED);
-
-  v4lmjpegsink->bufferpool = gst_buffer_pool_new(
-				  NULL, 
-				  NULL,
-				  (GstBufferPoolBufferNewFunction)gst_v4lmjpegsink_buffer_new,
-				  NULL,
-				  NULL,
-				  v4lmjpegsink);
 }
 
 
 static GstPadLinkReturn
 gst_v4lmjpegsink_sinkconnect (GstPad  *pad,
-                              GstCaps *vscapslist)
+                              const GstCaps *vscapslist)
 {
   GstV4lMjpegSink *v4lmjpegsink;
-  GstCaps *caps;
+  GstStructure *structure;
 
   v4lmjpegsink = GST_V4LMJPEGSINK (gst_pad_get_parent (pad));
 
-  /* we are not going to act on variable caps */
-  if (!GST_CAPS_IS_FIXED (vscapslist) || !GST_V4L_IS_OPEN(GST_V4LELEMENT(v4lmjpegsink)))
-    return GST_PAD_LINK_DELAYED;
-
   /* in case the buffers are active (which means that we already
    * did capsnego before and didn't clean up), clean up anyways */
   if (GST_V4L_IS_ACTIVE(GST_V4LELEMENT(v4lmjpegsink)))
     if (!gst_v4lmjpegsink_playback_deinit(v4lmjpegsink))
       return GST_PAD_LINK_REFUSED;
 
-  for (caps = vscapslist; caps != NULL; caps = vscapslist = vscapslist->next)
-  {
-    gst_caps_get_int (caps, "width", &v4lmjpegsink->width);
-    gst_caps_get_int (caps, "height", &v4lmjpegsink->height);
+  structure = gst_caps_get_structure (vscapslist, 0);
 
-    if (!gst_v4lmjpegsink_set_playback(v4lmjpegsink,
-         v4lmjpegsink->width, v4lmjpegsink->height,
-         v4lmjpegsink->x_offset, v4lmjpegsink->y_offset,
-         GST_V4LELEMENT(v4lmjpegsink)->norm, 0)) /* TODO: interlacing */
-      continue;
+  gst_structure_get_int  (structure, "width", &v4lmjpegsink->width);
+  gst_structure_get_int  (structure, "height", &v4lmjpegsink->height);
 
-    /* set buffer info */
-    if (!gst_v4lmjpegsink_set_buffer(v4lmjpegsink,
-         v4lmjpegsink->numbufs, v4lmjpegsink->bufsize))
-      continue;
-    if (!gst_v4lmjpegsink_playback_init(v4lmjpegsink))
-      continue;
+  if (!gst_v4lmjpegsink_set_playback(v4lmjpegsink,
+       v4lmjpegsink->width, v4lmjpegsink->height,
+       v4lmjpegsink->x_offset, v4lmjpegsink->y_offset,
+       GST_V4LELEMENT(v4lmjpegsink)->vchan.norm, 0)) /* TODO: interlacing */
+    return GST_PAD_LINK_REFUSED;
 
-    g_signal_emit (G_OBJECT (v4lmjpegsink), gst_v4lmjpegsink_signals[SIGNAL_HAVE_SIZE], 0,
-      v4lmjpegsink->width, v4lmjpegsink->height);
+  /* set buffer info */
+  if (!gst_v4lmjpegsink_set_buffer(v4lmjpegsink,
+       v4lmjpegsink->numbufs, v4lmjpegsink->bufsize))
+    return GST_PAD_LINK_REFUSED;
+  if (!gst_v4lmjpegsink_playback_init(v4lmjpegsink))
+    return GST_PAD_LINK_REFUSED;
 
-    return GST_PAD_LINK_OK;
-  }
+  return GST_PAD_LINK_OK;
 
-  /* if we got here - it's not good */
-  return GST_PAD_LINK_REFUSED;
 }
 
 
@@ -254,10 +239,10 @@ gst_v4lmjpegsink_set_clock (GstElement *element, GstClock *clock)
 
 static void
 gst_v4lmjpegsink_chain (GstPad    *pad,
-                        GstBuffer *buf)
+                        GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstV4lMjpegSink *v4lmjpegsink;
-  GstClockTimeDiff jitter;
   gint num;
 
   g_return_if_fail (pad != NULL);
@@ -267,20 +252,12 @@ gst_v4lmjpegsink_chain (GstPad    *pad,
   v4lmjpegsink = GST_V4LMJPEGSINK (gst_pad_get_parent (pad));
 
   if (v4lmjpegsink->clock) {
-    GstClockID id;
+    GST_DEBUG ("videosink: clock wait: %" G_GUINT64_FORMAT, GST_BUFFER_TIMESTAMP(buf));
 
-    GST_DEBUG (0,"videosink: clock wait: %llu", GST_BUFFER_TIMESTAMP(buf));
-
-    jitter = 0; /* FIXME: jitter = gst_clock_current_diff(v4lmjpegsink->clock, GST_BUFFER_TIMESTAMP (buf)); */
-
-    if (jitter > 500000 || jitter < -500000)
-      GST_DEBUG (0, "jitter: %lld", jitter);
-
-    id = gst_clock_new_single_shot_id (v4lmjpegsink->clock, GST_BUFFER_TIMESTAMP(buf));
-    gst_element_clock_wait(GST_ELEMENT(v4lmjpegsink), id, NULL);
-    gst_clock_id_free (id);
+    gst_element_wait (GST_ELEMENT(v4lmjpegsink), GST_BUFFER_TIMESTAMP(buf));
   }
 
+#if 0
   if (GST_BUFFER_POOL(buf) == v4lmjpegsink->bufferpool)
   {
     num = GPOINTER_TO_INT(GST_BUFFER_POOL_PRIVATE(buf));
@@ -288,12 +265,13 @@ gst_v4lmjpegsink_chain (GstPad    *pad,
   }
   else
   {
+#endif
     /* check size */
     if (GST_BUFFER_SIZE(buf) > v4lmjpegsink->breq.size)
     {
-      gst_element_error(GST_ELEMENT(v4lmjpegsink),
-        "Buffer too big (%d KB), max. buffersize is %d KB",
-        GST_BUFFER_SIZE(buf)/1024, v4lmjpegsink->breq.size/1024);
+      GST_ELEMENT_ERROR (v4lmjpegsink, RESOURCE, WRITE, (NULL),
+        ("Buffer too big (%d KB), max. buffersize is %ld KB",
+        GST_BUFFER_SIZE(buf)/1024, v4lmjpegsink->breq.size/1024));
       return;
     }
 
@@ -302,7 +280,9 @@ gst_v4lmjpegsink_chain (GstPad    *pad,
     memcpy(gst_v4lmjpegsink_get_buffer(v4lmjpegsink, num),
       GST_BUFFER_DATA(buf), GST_BUFFER_SIZE(buf));
     gst_v4lmjpegsink_play_frame(v4lmjpegsink, num);
+#if 0
   }
+#endif
 
   g_signal_emit(G_OBJECT(v4lmjpegsink),gst_v4lmjpegsink_signals[SIGNAL_FRAME_DISPLAYED],0);
 
@@ -310,6 +290,7 @@ gst_v4lmjpegsink_chain (GstPad    *pad,
 }
 
 
+#if 0
 static GstBuffer *
 gst_v4lmjpegsink_buffer_new (GstBufferPool *pool,
                              guint64        offset,
@@ -324,7 +305,7 @@ gst_v4lmjpegsink_buffer_new (GstBufferPool *pool,
   if (!GST_V4L_IS_ACTIVE(GST_V4LELEMENT(v4lmjpegsink)))
     return NULL;
   if (v4lmjpegsink->breq.size < size) {
-    GST_DEBUG(GST_CAT_PLUGIN_INFO, "Requested buffer size is too large (%d > %ld)",
+    GST_DEBUG ("Requested buffer size is too large (%d > %ld)",
       size, v4lmjpegsink->breq.size);
     return NULL;
   }
@@ -345,6 +326,7 @@ gst_v4lmjpegsink_buffer_new (GstBufferPool *pool,
 
   return buffer;
 }
+#endif
 
 
 static void
@@ -450,61 +432,8 @@ gst_v4lmjpegsink_change_state (GstElement *element)
     parent_value = GST_STATE_FAILURE;
   }
 
-  if (GST_STATE_TRANSITION(element) == GST_STATE_NULL_TO_READY)
-  {
-    if ((GST_V4LELEMENT(v4lmjpegsink)->norm >= VIDEO_MODE_PAL ||
-         GST_V4LELEMENT(v4lmjpegsink)->norm < VIDEO_MODE_AUTO) ||
-        GST_V4LELEMENT(v4lmjpegsink)->channel < 0)
-      if (!gst_v4l_set_chan_norm(GST_V4LELEMENT(v4lmjpegsink),
-           0, GST_V4LELEMENT(v4lmjpegsink)->norm))
-        return GST_STATE_FAILURE;
-  }
-
   if (GST_ELEMENT_CLASS (parent_class)->change_state)
     return parent_value;
 
   return GST_STATE_SUCCESS;
 }
-
-
-static gboolean
-plugin_init (GModule   *module,
-             GstPlugin *plugin)
-{
-  GstElementFactory *factory;
-  GstCaps *caps;
-
-  /* create an elementfactory for the v4lmjpegsink element */
-  factory = gst_element_factory_new("v4lmjpegsink",GST_TYPE_V4LMJPEGSINK,
-                                   &gst_v4lmjpegsink_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-
-  caps = gst_caps_new ("v4lmjpegsink_caps",
-                       "video/jpeg",
-                       gst_props_new (
-                          "width",  GST_PROPS_INT_RANGE (0, G_MAXINT),
-                          "height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-                          NULL       )
-                      );
-  capslist = gst_caps_append(capslist, caps);
-
-  sink_template = gst_pad_template_new (
-		  "sink",
-                  GST_PAD_SINK,
-  		  GST_PAD_ALWAYS,
-		  capslist, NULL);
-
-  gst_element_factory_add_pad_template (factory, sink_template);
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
-}
-
-
-GstPluginDesc plugin_desc = {
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "v4lmjpegsink",
-  plugin_init
-};
