@@ -147,7 +147,6 @@ gst_ximagesink_ximage_new (GstXImageSink *ximagesink, gint width, gint height)
 {
   GstXImage *ximage = NULL;
   
-  g_return_val_if_fail (ximagesink != NULL, NULL);
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
   
   ximage = g_new0 (GstXImage, 1);
@@ -179,6 +178,11 @@ gst_ximagesink_ximage_new (GstXImageSink *ximagesink, gint width, gint height)
       ximage->SHMInfo.readOnly = FALSE;
   
       XShmAttach (ximagesink->xcontext->disp, &ximage->SHMInfo);
+      
+      XSync (ximagesink->xcontext->disp, FALSE);
+      
+      shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
+      ximage->SHMInfo.shmid = -1;
     }
   else
 #endif /* HAVE_XSHM */
@@ -192,13 +196,11 @@ gst_ximagesink_ximage_new (GstXImageSink *ximagesink, gint width, gint height)
                                      ximage->width, ximage->height,
                                      ximagesink->xcontext->bpp,
                                      ximage->width * (ximagesink->xcontext->bpp / 8));
-    }
-  
-  if (ximage->ximage)
-    {
+      
       XSync (ximagesink->xcontext->disp, FALSE);
     }
-  else
+  
+  if (!ximage->ximage)
     {
       if (ximage->data)
         g_free (ximage->data);
@@ -218,7 +220,6 @@ static void
 gst_ximagesink_ximage_destroy (GstXImageSink *ximagesink, GstXImage *ximage)
 {
   g_return_if_fail (ximage != NULL);
-  g_return_if_fail (ximagesink != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
   
   /* If the destroyed image is the current one we destroy our reference too */
@@ -263,7 +264,6 @@ gst_ximagesink_ximage_put (GstXImageSink *ximagesink, GstXImage *ximage)
   gint x, y;
  
   g_return_if_fail (ximage != NULL);
-  g_return_if_fail (ximagesink != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
   
   /* Store a reference to the last image we put */
@@ -302,7 +302,6 @@ gst_ximagesink_xwindow_new (GstXImageSink *ximagesink, gint width, gint height)
 {
   GstXWindow *xwindow = NULL;
   
-  g_return_val_if_fail (ximagesink != NULL, NULL);
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
   
   xwindow = g_new0 (GstXWindow, 1);
@@ -341,7 +340,6 @@ static void
 gst_ximagesink_xwindow_destroy (GstXImageSink *ximagesink, GstXWindow *xwindow)
 {
   g_return_if_fail (xwindow != NULL);
-  g_return_if_fail (ximagesink != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
   
   g_mutex_lock (ximagesink->x_lock);
@@ -367,7 +365,6 @@ gst_ximagesink_xwindow_resize (GstXImageSink *ximagesink, GstXWindow *xwindow,
                                guint width, guint height)
 {
   g_return_if_fail (xwindow != NULL);
-  g_return_if_fail (ximagesink != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
   
   g_mutex_lock (ximagesink->x_lock);
@@ -377,27 +374,79 @@ gst_ximagesink_xwindow_resize (GstXImageSink *ximagesink, GstXWindow *xwindow,
   
   XResizeWindow (ximagesink->xcontext->disp, xwindow->win,
                  xwindow->width, xwindow->height);
-
+  
+  XSync (ximagesink->xcontext->disp, FALSE);
+  
   g_mutex_unlock (ximagesink->x_lock);
 }
 
 static void
-gst_ximagesink_renegotiate_size (GstXImageSink *ximagesink,
-                                 guint width, guint height)
+gst_ximagesink_xwindow_clear (GstXImageSink *ximagesink, GstXWindow *xwindow)
 {
-  g_return_if_fail (ximagesink != NULL);
+  g_return_if_fail (xwindow != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
   
-  /* Window got resized or moved. We do caps negotiation again to get video
-     scaler to fit that new size only if size of the window changed. */
+  g_mutex_lock (ximagesink->x_lock);
   
-  if ( (ximagesink->xwindow->width != width) ||
-       (ximagesink->xwindow->height != height) )
+  XSetForeground (ximagesink->xcontext->disp, xwindow->gc,
+                  ximagesink->xcontext->black);
+  
+  XFillRectangle (ximagesink->xcontext->disp, xwindow->win, xwindow->gc,
+                  0, 0, xwindow->width, xwindow->height);
+  
+  XSync (ximagesink->xcontext->disp, FALSE);
+  
+  g_mutex_unlock (ximagesink->x_lock);
+}
+
+static void
+gst_ximagesink_xwindow_update_geometry (GstXImageSink *ximagesink,
+                                        GstXWindow *xwindow)
+{
+  XWindowAttributes attr;
+  
+  g_return_if_fail (xwindow != NULL);
+  g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
+  
+  /* Update the window geometry */
+  g_mutex_lock (ximagesink->x_lock);
+  XGetWindowAttributes (ximagesink->xcontext->disp,
+                        ximagesink->xwindow->win, &attr);
+  g_mutex_unlock (ximagesink->x_lock);
+  
+  ximagesink->xwindow->width = attr.width;
+  ximagesink->xwindow->height = attr.height;
+}
+
+static void
+gst_ximagesink_renegotiate_size (GstXImageSink *ximagesink)
+{
+  g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
+  
+  if (!ximagesink->xwindow)
+    return;
+  
+  gst_ximagesink_xwindow_update_geometry (ximagesink, ximagesink->xwindow);
+  
+  if (ximagesink->sw_scaling_failed)
+    return;
+  
+  if (ximagesink->xwindow->width <= 1 || ximagesink->xwindow->height <= 1)
+    return;
+  
+  if (GST_PAD_IS_NEGOTIATING (GST_VIDEOSINK_PAD (ximagesink)) ||
+      !gst_pad_is_negotiated (GST_VIDEOSINK_PAD (ximagesink)))
+    return;
+  
+  /* Window got resized or moved. We do caps negotiation again to get video
+     scaler to fit that new size only if size of the window differs from our
+     size. */
+  
+  if (GST_VIDEOSINK_WIDTH (ximagesink) != ximagesink->xwindow->width ||
+      GST_VIDEOSINK_HEIGHT (ximagesink) != ximagesink->xwindow->height)
     {
       GstPadLinkReturn r;
-      ximagesink->xwindow->width = width;
-      ximagesink->xwindow->height = height;
-                
+      
       r = gst_pad_try_set_caps (GST_VIDEOSINK_PAD (ximagesink),
             gst_caps_new_simple ("video/x-raw-rgb",
               "bpp",        G_TYPE_INT, ximagesink->xcontext->bpp,
@@ -406,15 +455,16 @@ gst_ximagesink_renegotiate_size (GstXImageSink *ximagesink,
               "red_mask",   G_TYPE_INT, ximagesink->xcontext->visual->red_mask,
               "green_mask", G_TYPE_INT, ximagesink->xcontext->visual->green_mask,
               "blue_mask",  G_TYPE_INT, ximagesink->xcontext->visual->blue_mask,
-              "width",      G_TYPE_INT, width,
-              "height",     G_TYPE_INT, height,
+              "width",      G_TYPE_INT, ximagesink->xwindow->width,
+              "height",     G_TYPE_INT, ximagesink->xwindow->height,
               "framerate",  G_TYPE_DOUBLE, ximagesink->framerate,
               NULL));
 
       if ( (r == GST_PAD_LINK_OK) || (r == GST_PAD_LINK_DONE) )
         {
-          GST_VIDEOSINK_WIDTH (ximagesink) = width;
-          GST_VIDEOSINK_HEIGHT (ximagesink) = height;
+          /* Renegotiation succeeded, we update our size and image */
+          GST_VIDEOSINK_WIDTH (ximagesink) = ximagesink->xwindow->width;
+          GST_VIDEOSINK_HEIGHT (ximagesink) = ximagesink->xwindow->height;
 
           if ( (ximagesink->ximage) &&
                ( (GST_VIDEOSINK_WIDTH (ximagesink) != ximagesink->ximage->width) ||
@@ -428,6 +478,10 @@ gst_ximagesink_renegotiate_size (GstXImageSink *ximagesink,
                                              GST_VIDEOSINK_HEIGHT (ximagesink));
             }
         }
+      else
+        {
+          ximagesink->sw_scaling_failed = TRUE;
+        }
     }
 }
 
@@ -439,41 +493,12 @@ static void
 gst_ximagesink_handle_xevents (GstXImageSink *ximagesink, GstPad *pad)
 {
   XEvent e;
-  guint new_width = 0, new_height = 0, pointer_x = 0, pointer_y = 0;
+  guint pointer_x = 0, pointer_y = 0;
   gboolean pointer_moved = FALSE;
   
-  g_return_if_fail (ximagesink != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
   
-  /* First we get all structure modification events. Only the last one is 
-     interesting */
-  g_mutex_lock (ximagesink->x_lock);
-  while (XCheckWindowEvent (ximagesink->xcontext->disp,
-                            ximagesink->xwindow->win,
-                            StructureNotifyMask, &e))
-    {
-      g_mutex_unlock (ximagesink->x_lock);
-      
-      switch (e.type)
-        {
-          case ConfigureNotify:
-            new_width = e.xconfigure.width;
-            new_height = e.xconfigure.height;
-            break;
-          default:
-            break;
-        }
-        
-      g_mutex_lock (ximagesink->x_lock);
-    }
-  g_mutex_unlock (ximagesink->x_lock);
-    
-  if ( (new_width != 0) && (new_height != 0) )
-    {
-      GST_DEBUG ("ximagesink window geometry is : %d,%d",
-                 new_width, new_height);
-      gst_ximagesink_renegotiate_size (ximagesink, new_width, new_height);
-    }
+  gst_ximagesink_renegotiate_size (ximagesink);
   
   /* Then we get all pointer motion events, only the last position is
      interesting. */
@@ -580,7 +605,6 @@ gst_ximagesink_xcontext_get (GstXImageSink *ximagesink)
   XPixmapFormatValues *px_formats = NULL;
   gint nb_formats = 0, i;
   
-  g_return_val_if_fail (ximagesink != NULL, NULL);
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
   
   xcontext = g_new0 (GstXContext, 1);
@@ -679,7 +703,6 @@ gst_ximagesink_xcontext_get (GstXImageSink *ximagesink)
 static void
 gst_ximagesink_xcontext_clear (GstXImageSink *ximagesink)
 {
-  g_return_if_fail (ximagesink != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
   
   gst_caps_free (ximagesink->xcontext->caps);
@@ -842,6 +865,7 @@ gst_ximagesink_change_state (GstElement *element)
         return GST_STATE_FAILURE;
       break;
     case GST_STATE_READY_TO_PAUSED:
+      gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
       ximagesink->time = 0;
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
@@ -850,6 +874,7 @@ gst_ximagesink_change_state (GstElement *element)
       break;
     case GST_STATE_PAUSED_TO_READY:
       ximagesink->framerate = 0;
+      ximagesink->sw_scaling_failed = FALSE;
       GST_VIDEOSINK_WIDTH (ximagesink) = 0;
       GST_VIDEOSINK_HEIGHT (ximagesink) = 0;
       break;
@@ -889,7 +914,6 @@ gst_ximagesink_chain (GstPad *pad, GstData *data)
   GstBuffer *buf = GST_BUFFER (data);
   GstXImageSink *ximagesink;
   
-  g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (buf != NULL);
 
@@ -1095,7 +1119,6 @@ gst_ximagesink_set_xwindow_id (GstXOverlay *overlay, XID xwindow_id)
   GstXWindow *xwindow = NULL;
   XWindowAttributes attr;
   
-  g_return_if_fail (ximagesink != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
   
   /* If we already use that window return */
@@ -1222,6 +1245,18 @@ gst_ximagesink_expose (GstXOverlay *overlay)
 {
   GstXImageSink *ximagesink = GST_XIMAGESINK (overlay);
   
+  if (!ximagesink->xwindow)
+    return;
+  
+  gst_ximagesink_xwindow_update_geometry (ximagesink, ximagesink->xwindow);
+  
+  /* We don't act on internal window from outside that could cause some thread
+     race with the video sink own thread checking for configure event */
+  if (ximagesink->xwindow->internal)
+    return;
+  
+  gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
+  
   if (ximagesink->cur_image)
     gst_ximagesink_ximage_put (ximagesink, ximagesink->cur_image);
 }
@@ -1346,6 +1381,9 @@ gst_ximagesink_init (GstXImageSink *ximagesink)
   
   ximagesink->image_pool = NULL;
   ximagesink->pool_lock = g_mutex_new ();
+  
+  ximagesink->sw_scaling_failed = FALSE;
+  ximagesink->synchronous = FALSE;
 
   GST_FLAG_SET(ximagesink, GST_ELEMENT_THREAD_SUGGESTED);
   GST_FLAG_SET(ximagesink, GST_ELEMENT_EVENT_AWARE);
