@@ -231,16 +231,16 @@ _gst_ximage_add_formats (GstXWindow *window)
   }
 
   image_format->type = GST_TYPE_XIMAGE;
-  image_format->caps = GST_CAPS_NEW("xvideosink_ximage_caps", "video/x-raw-rgb",
-      "bpp", GST_PROPS_INT (GST_XIMAGE_BPP (ximage)),
-      "depth",      GST_PROPS_INT (GST_XIMAGE_DEPTH (ximage)),
-      "endianness", GST_PROPS_INT (GST_XIMAGE_ENDIANNESS (ximage)),
-      "red_mask",   GST_PROPS_INT (GST_XIMAGE_RED_MASK (ximage)),
-      "green_mask", GST_PROPS_INT (GST_XIMAGE_GREEN_MASK (ximage)),
-      "blue_mask",  GST_PROPS_INT (GST_XIMAGE_BLUE_MASK (ximage)),
-      "width",      GST_PROPS_INT_RANGE (0, G_MAXINT),
-      "height",     GST_PROPS_INT_RANGE (0, G_MAXINT),
-      "framerate",  GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+  image_format->caps = gst_caps_new_simple ("video/x-raw-rgb",
+      "bpp", G_TYPE_INT, GST_XIMAGE_BPP (ximage),
+      "depth",      G_TYPE_INT, GST_XIMAGE_DEPTH (ximage),
+      "endianness", G_TYPE_INT, GST_XIMAGE_ENDIANNESS (ximage),
+      "red_mask",   G_TYPE_INT, GST_XIMAGE_RED_MASK (ximage),
+      "green_mask", G_TYPE_INT, GST_XIMAGE_GREEN_MASK (ximage),
+      "blue_mask",  G_TYPE_INT, GST_XIMAGE_BLUE_MASK (ximage),
+      "width",      GST_TYPE_INT_RANGE, 0, G_MAXINT,
+      "height",     GST_TYPE_INT_RANGE, 0, G_MAXINT,
+      "framerate",  GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXFLOAT, NULL);
 
   _gst_image_destroy (GST_IMAGE (ximage));
 
