@@ -43,6 +43,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#include <string.h>
 
 #if defined (HAVE_IPC_H) && defined (HAVE_XSHM_H) && defined (HAVE_XSHM_H)
 #define HAVE_XSHM
@@ -58,9 +59,6 @@
 #include <X11/Xatom.h>
 #include <X11/Xproto.h>
 #include <X11/Xmd.h>
-#include <X11/Intrinsic.h>
-#include <X11/StringDefs.h>
-#include <X11/Shell.h>
 
 #ifdef HAVE_XSHM
 #include <X11/extensions/XShm.h>
@@ -412,17 +410,16 @@ _gst_xvimage_add_formats(GstXWindow *window)
 	    fo[j].red_mask,
 	    fo[j].green_mask,
 	    fo[j].blue_mask);
-        image_format->caps = GST_CAPS_NEW ( "xvideosink_caps",
-	    "video/x-raw-rgb", 
-	    "endianness", GST_PROPS_INT (G_BYTE_ORDER),
-	    "depth",     GST_PROPS_INT (fo[j].depth),
-	    "bpp",       GST_PROPS_INT (fo[j].bits_per_pixel),
-	    "blue_mask", GST_PROPS_INT (fo[j].red_mask),
-	    "green_mask", GST_PROPS_INT (fo[j].green_mask),
-	    "red_mask",  GST_PROPS_INT (fo[j].blue_mask),
-            "width",     GST_PROPS_INT_RANGE (0, G_MAXINT),
-	    "height",    GST_PROPS_INT_RANGE (0, G_MAXINT),
-	    "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+        image_format->caps = gst_caps_new_simple ("video/x-raw-rgb", 
+	    "endianness", G_TYPE_INT, G_BYTE_ORDER,
+	    "depth",     G_TYPE_INT, fo[j].depth,
+	    "bpp",       G_TYPE_INT, fo[j].bits_per_pixel,
+	    "blue_mask", G_TYPE_INT, fo[j].red_mask,
+	    "green_mask", G_TYPE_INT, fo[j].green_mask,
+	    "red_mask",  G_TYPE_INT, fo[j].blue_mask,
+            "width",     GST_TYPE_INT_RANGE, 0, G_MAXINT,
+	    "height",    GST_TYPE_INT_RANGE, 0, G_MAXINT,
+	    "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
       }else{
       	GST_DEBUG ( "    YUV 0x%x (" GST_FOURCC_FORMAT ") %s %.32s (%d:%d;%d,%d:%d:%d,%d:%d:%d)",
 	    fo[j].id,
@@ -439,12 +436,12 @@ _gst_xvimage_add_formats(GstXWindow *window)
 	    fo[j].vert_u_period,
 	    fo[j].vert_v_period);
 
-        image_format->caps = GST_CAPS_NEW ( "xvideosink_caps",
-	    "video/x-raw-yuv", 
-	    "format",      GST_PROPS_FOURCC (fo[j].id),
-            "width",     GST_PROPS_INT_RANGE (0, G_MAXINT),
-	    "height",    GST_PROPS_INT_RANGE (0, G_MAXINT),
-	    "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
+        image_format->caps = gst_caps_new_simple ("video/x-raw-yuv", 
+	    "format",    GST_TYPE_FOURCC, fo[j].id,
+            "width",     GST_TYPE_INT_RANGE, 0, G_MAXINT,
+	    "height",    GST_TYPE_INT_RANGE, 0, G_MAXINT,
+	    "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE,
+	    NULL);
       }
 
       g_ptr_array_add(window->image_formats, image_format);
