@@ -82,19 +82,22 @@ static void		gst_xvideosink_image_free	(GstXVideoSink *xvideosink);
 
 /* default template - initiated with class struct to allow gst-register to work
    with X running */
-GST_PAD_TEMPLATE_FACTORY (gst_xvideosink_sink_template_factory,
+static GstStaticPadTemplate gst_xvideosink_sink_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW ( "videosink_rgbsink", "video/x-raw-rgb",
-    "framerate", GST_PROPS_FLOAT_RANGE(0, G_MAXFLOAT),
-    "width", GST_PROPS_INT_RANGE(0, G_MAXINT),
-    "height", GST_PROPS_INT_RANGE(0, G_MAXINT)),
-  GST_CAPS_NEW ( "videosink_yuvsink", "video/x-raw-yuv",
-    "framerate", GST_PROPS_FLOAT_RANGE(0, G_MAXFLOAT),
-    "width", GST_PROPS_INT_RANGE(0, G_MAXINT),
-    "height", GST_PROPS_INT_RANGE(0, G_MAXINT))
-)
+  GST_STATIC_CAPS (
+    "video/x-raw-rgb, "
+      "framerate = (double) [ 0, MAX ], "
+      "width = (int) [ 1, MAX ], "
+      "height = (int) [ 1, MAX ]; "
+    "video/x-raw-yuv, "
+      "framerate = (double) [ 0, MAX ], "
+      "width = (int) [ 1, MAX ], "
+      "height = (int) [ 1, MAX ]"
+  )
+);
 
 static GstElementClass *parent_class = NULL;
 
@@ -151,7 +154,7 @@ gst_xvideosink_base_init (gpointer g_class)
   gst_element_class_set_details (element_class, &gst_xvideosink_details);
 
   gst_element_class_add_pad_template (element_class, 
-    GST_PAD_TEMPLATE_GET (gst_xvideosink_sink_template_factory));
+    gst_static_pad_template_get (&gst_xvideosink_sink_template_factory));
 }
 static void
 gst_xvideosink_class_init (GstXVideoSinkClass *klass)
@@ -265,6 +268,7 @@ gst_xvideosink_imagepool_clear (GstXVideoSink *xvideosink)
   g_mutex_unlock(xvideosink->pool_lock);
 }
 
+#if 0
 G_GNUC_UNUSED static void
 gst_xvideosink_buffer_pool_free (GstBufferPool *pool)
 {
@@ -274,7 +278,9 @@ gst_xvideosink_buffer_pool_free (GstBufferPool *pool)
 
   gst_buffer_pool_default_free (pool);
 }
+#endif
 
+#if 0
 static GstBuffer*
 gst_xvideosink_buffer_new (GstBufferPool *pool,  
 		           gint64 location, guint size, gpointer user_data)
@@ -323,7 +329,7 @@ gst_xvideosink_buffer_new (GstBufferPool *pool,
   }
 
   if (image == NULL) {
-    gst_element_error(GST_ELEMENT (xvideosink), "image creation failed");
+    GST_ELEMENT_ERROR (xvideosink, RESOURCE, TOO_LAZY, (NULL), ("image creation failed"));
     /* FIXME: need better error handling here */
     return NULL;
   }
@@ -335,7 +341,9 @@ gst_xvideosink_buffer_new (GstBufferPool *pool,
 
   return buffer;
 }
+#endif
 
+#if 0
 static void
 gst_xvideosink_buffer_free (GstBufferPool *pool, GstBuffer *buffer, gpointer user_data)
 {
@@ -381,32 +389,7 @@ gst_xvideosink_buffer_free (GstBufferPool *pool, GstBuffer *buffer, gpointer use
 
   gst_buffer_default_free (buffer);
 }
-
-static GstBufferPool*
-gst_xvideosink_get_bufferpool (GstPad *pad)
-{
-  GstXVideoSink *xvideosink;
-  
-  xvideosink = GST_XVIDEOSINK (gst_pad_get_parent (pad));
-
-  if (!xvideosink->bufferpool) {
-    GST_DEBUG ("xvideosink: creating bufferpool");
-
-    xvideosink->bufferpool = gst_buffer_pool_new (
-		    NULL,		/* free */
-		    NULL,		/* copy */
-		    (GstBufferPoolBufferNewFunction)gst_xvideosink_buffer_new,
-		    NULL,		/* buffer copy, the default is fine */
-		    (GstBufferPoolBufferFreeFunction)gst_xvideosink_buffer_free,
-		    xvideosink);
-
-    xvideosink->image_pool = NULL;
-  }
-
-  gst_buffer_pool_ref (xvideosink->bufferpool);
-
-  return xvideosink->bufferpool;
-}
+#endif
 
 static void
 gst_xvideosink_get_real_size (GstXVideoSink *xvideosink, 
@@ -430,23 +413,17 @@ gst_xvideosink_get_real_size (GstXVideoSink *xvideosink,
 }
 
 static GstPadLinkReturn
-gst_xvideosink_sinkconnect (GstPad *pad, GstCaps *caps)
+gst_xvideosink_sinkconnect (GstPad *pad, const GstCaps *caps)
 {
   GstXVideoSink *xvideosink;
   GstCaps *icaps;
   int i;
   GstImageFormat *image_format;
+  GstStructure *structure;
 
   xvideosink = GST_XVIDEOSINK (gst_pad_get_parent (pad));
 
-  /* we are not going to act on variable caps */
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
-
-  if (GST_CAPS_IS_CHAINED (caps))
-    return GST_PAD_LINK_DELAYED;
-
-  GST_DEBUG ("sinkconnect %s", gst_caps_to_string(caps));
+  GST_DEBUG ("sinkconnect: " GST_PTR_FORMAT, caps);
 
   image_format = NULL;
   icaps = NULL;
@@ -458,37 +435,33 @@ gst_xvideosink_sinkconnect (GstPad *pad, GstCaps *caps)
 
     icaps = gst_caps_intersect (caps, image_format->caps);
 
-    if(icaps != NULL){
+    if(!gst_caps_is_empty(icaps)) {
       break;
     }
+    gst_caps_free (icaps);
   }
 
-  if(icaps == NULL){
+  if (i == xvideosink->window->image_formats->len) {
     GST_DEBUG ("no format found");
     return GST_PAD_LINK_REFUSED;
   }
 
-  gst_caps_unref(icaps);
+  gst_caps_free (icaps);
 
   GST_DEBUG ("using format %d",i);
   xvideosink->image_format = image_format;
 
-  gst_caps_get_int (caps, "width",
+  structure = gst_caps_get_structure (caps, 0);
+  gst_structure_get_int (structure, "width",
                     &(GST_VIDEOSINK_WIDTH (xvideosink)));
-  gst_caps_get_int (caps, "height",
+  gst_structure_get_int (structure, "height",
                     &(GST_VIDEOSINK_HEIGHT (xvideosink)));
 
-  if (gst_caps_has_fixed_property (caps, "pixel_width")) {
-    gst_caps_get_int (caps, "pixel_width", &xvideosink->pixel_width);
-  } else {
-    xvideosink->pixel_width = 1;
-  }
+  xvideosink->pixel_width = 1;
+  gst_structure_get_int  (structure, "pixel_width", &xvideosink->pixel_width);
 
-  if (gst_caps_has_fixed_property (caps, "pixel_height")) {
-    gst_caps_get_int (caps, "pixel_height", &xvideosink->pixel_height);
-  } else {
-    xvideosink->pixel_height = 1;
-  }
+  xvideosink->pixel_height = 1;
+  gst_structure_get_int  (structure, "pixel_height", &xvideosink->pixel_height);
 
   g_mutex_lock (xvideosink->lock);
   if (!gst_xvideosink_image_new (xvideosink)) {
@@ -500,7 +473,7 @@ gst_xvideosink_sinkconnect (GstPad *pad, GstCaps *caps)
   if (xvideosink->image == NULL) {
     /* FIXME: need better error handling? */
     g_mutex_unlock (xvideosink->lock);
-    gst_element_error (GST_ELEMENT (xvideosink), "image creation failed");
+    GST_ELEMENT_ERROR (xvideosink, RESOURCE, TOO_LAZY, (NULL), ("image creation failed"));
     xvideosink->image_format = NULL;
     return GST_PAD_LINK_REFUSED;
   }
@@ -515,16 +488,17 @@ gst_xvideosink_sinkconnect (GstPad *pad, GstCaps *caps)
     if (xvideosink->auto_size) {
       _gst_xwindow_resize (xvideosink->window, real_x, real_y);
     }
-    gst_video_sink_got_video_size (GST_VIDEOSINK (xvideosink), real_x, real_y);
+    gst_x_overlay_got_desired_size (GST_X_OVERLAY (xvideosink), real_x, real_y);
   }
 
   return GST_PAD_LINK_OK;
 }
 
 static GstCaps *
-gst_xvideosink_getcaps (GstPad *pad, GstCaps *caps)
+gst_xvideosink_getcaps (GstPad *pad)
 {
   GstXVideoSink *xvideosink;
+  GstCaps *caps;
 
   xvideosink = GST_XVIDEOSINK (gst_pad_get_parent (pad));
 
@@ -533,25 +507,25 @@ gst_xvideosink_getcaps (GstPad *pad, GstCaps *caps)
     GPtrArray *image_formats;
     GstImageFormat *image_format;
 
-    caps = NULL;
+    caps = gst_caps_new_empty ();
     image_formats = xvideosink->window->image_formats;
     for(i=0;i<image_formats->len;i++){
       image_format = g_ptr_array_index(image_formats, i);
-      caps = gst_caps_append(caps, gst_caps_copy(image_format->caps));
+      gst_caps_append (caps, gst_caps_copy (image_format->caps));
     }
   }else{
-    caps = gst_caps_append(
-	GST_CAPS_NEW ( "videosink_rgbsink", "video/x-raw-rgb",
-	  "framerate", GST_PROPS_FLOAT_RANGE(0, G_MAXFLOAT),
-	  "width", GST_PROPS_INT_RANGE(0, G_MAXINT),
-	  "height", GST_PROPS_INT_RANGE(0, G_MAXINT)),
-	GST_CAPS_NEW ( "videosink_yuvsink", "video/x-raw-yuv",
-	  "framerate", GST_PROPS_FLOAT_RANGE(0, G_MAXFLOAT),
-	  "width", GST_PROPS_INT_RANGE(0, G_MAXINT),
-	  "height", GST_PROPS_INT_RANGE(0, G_MAXINT)));
+    caps = gst_caps_from_string (
+	"video/x-raw-rgb, "
+	  "framerate = (double) [ 0, MAX ], "
+	  "width = (int) [ 1, MAX ], "
+	  "height = (int) [ 1, MAX ]; "
+	"video/x-raw-yuv, "
+	  "framerate = (double) [ 0, MAX ], "
+	  "width = (int) [ 1, MAX ], "
+	  "height = (int) [ 1, MAX ]");
   }
 
-  GST_DEBUG ("getcaps returning %s",gst_caps_to_string(caps));
+  GST_DEBUG ("getcaps returning: " GST_PTR_FORMAT, caps);
 
   return caps;
 }
@@ -560,7 +534,7 @@ static void
 gst_xvideosink_init (GstXVideoSink *xvideosink)
 {
   GST_VIDEOSINK_PAD (xvideosink) = gst_pad_new_from_template (
-                    GST_PAD_TEMPLATE_GET (gst_xvideosink_sink_template_factory), 
+                    gst_static_pad_template_get (&gst_xvideosink_sink_template_factory), 
                     "sink");
   gst_element_add_pad (GST_ELEMENT (xvideosink),
                        GST_VIDEOSINK_PAD (xvideosink));
@@ -570,8 +544,6 @@ gst_xvideosink_init (GstXVideoSink *xvideosink)
                              gst_xvideosink_sinkconnect);
   gst_pad_set_getcaps_function (GST_VIDEOSINK_PAD (xvideosink),
                                 gst_xvideosink_getcaps);
-  gst_pad_set_bufferpool_function (GST_VIDEOSINK_PAD (xvideosink),
-                                   gst_xvideosink_get_bufferpool);
 
   /* one-time initialization */
   xvideosink->window = NULL;
@@ -614,9 +586,6 @@ gst_xvideosink_dispose (GObject *object)
   g_mutex_free (xvideosink->lock);
   g_mutex_free (xvideosink->pool_lock);
 
-  if (xvideosink->bufferpool) 
-    gst_buffer_pool_free (xvideosink->bufferpool);
-
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -636,46 +605,30 @@ gst_xvideosink_chain (GstPad *pad, GstData *_data)
 
   if (GST_IS_EVENT (buf)) {
     GstEvent *event = GST_EVENT (buf);
-    gint64 offset;
-
-    switch (GST_EVENT_TYPE (event)) {
-      case GST_EVENT_DISCONTINUOUS:
-	offset = GST_EVENT_DISCONT_OFFSET (event, 0).value;
-	GST_DEBUG ("xvideo discont %" G_GINT64_FORMAT, offset);
-	gst_clock_handle_discont (GST_VIDEOSINK_CLOCK (xvideosink),
-                                  (guint64) offset);
-	break;
-      default:
-	gst_pad_event_default (pad, event);
-	break;
-    }
+    
+    gst_pad_event_default (pad, event);
     gst_event_unref (event);
     return;
   }
 
   if (GST_VIDEOSINK_CLOCK (xvideosink) && time != -1) {
-    GstClockReturn ret;
-
-    xvideosink->id = gst_clock_new_single_shot_id (
-                       GST_VIDEOSINK_CLOCK (xvideosink), time);
-
     GST_DEBUG ("videosink: clock %s wait: %" G_GUINT64_FORMAT " %u", 
                GST_OBJECT_NAME (GST_VIDEOSINK_CLOCK (xvideosink)),
                time, GST_BUFFER_SIZE (buf));
 
-    ret = gst_clock_id_wait (xvideosink->id, &jitter);
-    gst_clock_id_free (xvideosink->id);
-    xvideosink->id = NULL;
+    gst_element_wait (GST_ELEMENT (xvideosink), time);
   }
   if (GST_VIDEOSINK_CLOCK (xvideosink))
     time = gst_clock_get_time (GST_VIDEOSINK_CLOCK (xvideosink));
 
   g_mutex_lock (xvideosink->lock);
+#if 0
   /* if we have a pool and the image is from this pool, simply _put it */
   if (xvideosink->bufferpool && GST_BUFFER_BUFFERPOOL (buf) == xvideosink->bufferpool) {
     _gst_image_put (xvideosink->window, GST_IMAGE (GST_BUFFER_POOL_PRIVATE (buf)));
   }
   else {
+#endif
     /* else we have to copy the data into our private image, if we have one... */
     if (xvideosink->image) {
       memcpy (GST_IMAGE_DATA (xvideosink->image), 
@@ -687,10 +640,12 @@ gst_xvideosink_chain (GstPad *pad, GstData *_data)
     else {
       g_mutex_unlock (xvideosink->lock);
       gst_buffer_unref (buf);
-      gst_element_error (GST_ELEMENT (xvideosink), "no image to draw (are you feeding me video?)");
+      GST_ELEMENT_ERROR (xvideosink, CORE, NEGOTIATION, (NULL), ("no format defined before chain function"));
       return;
     }
+#if 0
   }
+#endif
 
   g_mutex_unlock (xvideosink->lock);
 
@@ -790,7 +745,7 @@ gst_xvideosink_release_locks (GstElement *element)
   xvideosink = GST_XVIDEOSINK (element);
 
   if (xvideosink->id) {
-    gst_clock_id_unlock (xvideosink->id);
+    //FIXME: gst_clock_id_unlock (xvideosink->id);
   }
 
   return TRUE;
@@ -803,13 +758,13 @@ gst_xvideosink_xwindow_new (GstXVideoSink *xvideosink)
 	                                 GST_VIDEOSINK_HEIGHT (xvideosink), 
 					 xvideosink->xwindow_id);
   if (!xvideosink->window) {
-    gst_element_error (GST_ELEMENT (xvideosink), "could not create X window");
+    GST_ELEMENT_ERROR (xvideosink, RESOURCE, TOO_LAZY, (NULL), ("could not create X window"));
     return FALSE;
   }
   
-  gst_video_sink_got_video_size (GST_VIDEOSINK (xvideosink),
-                                 GST_VIDEOSINK_WIDTH (xvideosink),
-                                 GST_VIDEOSINK_HEIGHT (xvideosink));
+  gst_x_overlay_got_desired_size (GST_X_OVERLAY (xvideosink),
+                                  GST_VIDEOSINK_WIDTH (xvideosink),
+                                  GST_VIDEOSINK_HEIGHT (xvideosink));
   
   if (!xvideosink->image_format) {
     return TRUE;
@@ -821,11 +776,6 @@ gst_xvideosink_xwindow_new (GstXVideoSink *xvideosink)
 static void
 gst_xvideosink_xwindow_free (GstXVideoSink *xvideosink)
 {
-  if (xvideosink->bufferpool) {
-    gst_buffer_pool_unref (xvideosink->bufferpool);
-    xvideosink->bufferpool = NULL;
-  }
-
   gst_xvideosink_image_free (xvideosink);
   if (xvideosink->window) {  
     _gst_xwindow_destroy (xvideosink->window);
